#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import MAR, MAY, JUN, JUL, AUG, SEP, OCT, NOV
from holidays.constants import HALF_DAY, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Italy(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Italy holidays.

    References:
        * <https://it.wikipedia.org/wiki/Festività_in_Italia>
        * <https://web.archive.org/web/20260115032712/https://www.normattiva.it/>
        * [Royal Decree 5342 of Oct 17, 1869](https://web.archive.org/web/20240405131328/https://www.gazzettaufficiale.it/eli/gu/1869/11/23/320/sg/pdf)
        * [Law 1968 of Jun 23, 1874](https://web.archive.org/web/20240101200400/https://www.gazzettaufficiale.it/eli/gu/1874/07/11/164/sg/pdf)
        * [Law 401 of Jul 19, 1895](https://web.archive.org/web/20240405131313/https://www.gazzettaufficiale.it/eli/gu/1895/07/19/169/sg/pdf)
        * [Royal Decree 1027 of Aug 4, 1913](https://web.archive.org/web/20240404233907/https://www.gazzettaufficiale.it/eli/gu/1913/09/10/211/sg/pdf)
        * [Royal Decree-Law 1354 of Oct 23, 1922](https://web.archive.org/web/20240101212042/https://www.gazzettaufficiale.it/eli/gu/1922/10/26/252/sg/pdf)
        * [Royal Decree-Law 833 of Apr 19, 1923](https://web.archive.org/web/20240423171524/https://www.gazzettaufficiale.it/eli/gu/1923/04/20/93/sg/pdf)
        * [Royal Decree-Law 2859 of Dec 30, 1923](https://web.archive.org/web/20250125021721/https://www.gazzettaufficiale.it/eli/gu/1924/01/15/12/sg/pdf)
        * [Royal Decree-Law 1779 of Oct 21, 1926](https://web.archive.org/web/20221021200745/https://www.gazzettaufficiale.it/eli/gu/1926/10/23/247/sg/pdf)
        * [Royal Decree-Law 1922 of Oct 23, 1927](https://web.archive.org/web/20250124092407/https://www.gazzettaufficiale.it/eli/gu/1927/10/25/247/sg/pdf)
        * [Law 2765 of Dec 6, 1928](https://web.archive.org/web/20251026200018/https://www.gazzettaufficiale.it/eli/gu/1928/12/19/294/sg/pdf)
        * [Royal Decree-Law 1827 of Oct 13, 1929](https://web.archive.org/web/20250124071534/https://www.gazzettaufficiale.it/eli/gu/1929/10/22/246/sg/pdf)
        * [Law 1726 of Dec 27, 1930](https://web.archive.org/web/20240406172031/https://www.gazzettaufficiale.it/eli/gu/1931/01/13/9/sg/pdf)
        * [Law 661 of May 5, 1939](https://web.archive.org/web/20240405131312/https://www.gazzettaufficiale.it/eli/gu/1939/05/08/109/sg/pdf)
        * [Royal Decree-Law 781 of Jul 24, 1941](https://web.archive.org/web/20250123213649/https://www.gazzettaufficiale.it/eli/gu/1941/08/16/192/sg/pdf)
        * [Legislative Decree 185 of Apr 22, 1946](https://web.archive.org/web/20230424155628/http://www.gazzettaufficiale.it/eli/gu/1946/04/24/96/sg/pdf)
        * [Legislative Decree 387 of May 28, 1947](https://web.archive.org/web/20231117073456/http://www.gazzettaufficiale.it/eli/gu/1947/05/31/123/sg/pdf)
        * [Law 260 of May 27, 1949](https://web.archive.org/web/20250806020941/https://www.gazzettaufficiale.it/eli/gu/1949/05/31/124/sg/pdf)
        * [Law 54 of Mar 5, 1977](https://web.archive.org/web/20250426211938/https://www.gazzettaufficiale.it/eli/gu/1977/03/07/63/sg/pdf)
        * [Presidential Decree 792 of Dec 28, 1985](https://web.archive.org/web/20230421223502/https://www.gazzettaufficiale.it/eli/gu/1985/12/31/306/sg/pdf)
        * [Law 336 of Nov 20, 2000](https://web.archive.org/web/20230604111415/https://www.gazzettaufficiale.it/eli/gu/2000/11/22/273/sg/pdf)
        * [Law 151 of Oct 8, 2025](https://web.archive.org/web/20251027040938/https://www.gazzettaufficiale.it/eli/gu/2025/10/10/236/sg/pdf)
        * [Provinces holidays](https://it.wikipedia.org/wiki/Santi_patroni_cattolici_delle_città_capoluogo_di_provincia_italiane)
        * [Bolzano Province Law 36 of Oct 16, 1992](https://web.archive.org/web/20260121223715/https://www.edizionieuropee.it/LAW/HTML/103/bz2_01_100.html)
        * [Current List of Italy Public Holidays](https://web.archive.org/web/20260311130312/https://www.italia.it/it/italia/informazioni/orari-e-festivita-in-italia)
    """

    country = "IT"
    default_language = "it_IT"
    # Royal Decree 5342 of Oct 17, 1869.
    start_year = 1870
    subdivisions = (
        # Provinces.
        "AG",  # Agrigento.
        "AL",  # Alessandria.
        "AN",  # Ancona.
        "AO",  # Aosta (deprecated).
        "AP",  # Ascoli Piceno.
        "AQ",  # L'Aquila.
        "AR",  # Arezzo.
        "AT",  # Asti.
        "AV",  # Avellino.
        "BA",  # Bari.
        "BG",  # Bergamo.
        "BI",  # Biella.
        "BL",  # Belluno.
        "BN",  # Benevento.
        "BO",  # Bologna.
        "BR",  # Brindisi.
        "BS",  # Brescia.
        "BT",  # Barletta-Andria-Trani.
        "BZ",  # Bolzano.
        "CA",  # Cagliari.
        "CB",  # Campobasso.
        "CE",  # Caserta.
        "CH",  # Chieti.
        "CL",  # Caltanissetta.
        "CN",  # Cuneo.
        "CO",  # Como.
        "CR",  # Cremona.
        "CS",  # Cosenza.
        "CT",  # Catania.
        "CZ",  # Catanzaro.
        "EN",  # Enna.
        "FC",  # Forlì-Cesena.
        "FE",  # Ferrara.
        "FG",  # Foggia.
        "FI",  # Firenze.
        "FM",  # Fermo.
        "FR",  # Frosinone.
        "GE",  # Genova.
        "GO",  # Gorizia.
        "GR",  # Grosseto.
        "IM",  # Imperia.
        "IS",  # Isernia.
        "KR",  # Crotone.
        "LC",  # Lecco.
        "LE",  # Lecce.
        "LI",  # Livorno.
        "LO",  # Lodi.
        "LT",  # Latina.
        "LU",  # Lucca.
        "MB",  # Monza e Brianza.
        "MC",  # Macerata.
        "ME",  # Messina.
        "MI",  # Milano.
        "MN",  # Mantova.
        "MO",  # Modena.
        "MS",  # Massa-Carrara.
        "MT",  # Matera.
        "NA",  # Napoli.
        "NO",  # Novara.
        "NU",  # Nuoro.
        "OR",  # Oristano.
        "PA",  # Palermo.
        "PC",  # Piacenza.
        "PD",  # Padova.
        "PE",  # Pescara.
        "PG",  # Perugia.
        "PI",  # Pisa.
        "PN",  # Pordenone.
        "PO",  # Prato.
        "PR",  # Parma.
        "PT",  # Pistoia.
        "PU",  # Pesaro e Urbino.
        "PV",  # Pavia.
        "PZ",  # Potenza.
        "RA",  # Ravenna.
        "RC",  # Reggio Calabria.
        "RE",  # Reggio Emilia.
        "RG",  # Ragusa.
        "RI",  # Rieti.
        "RM",  # Roma.
        "RN",  # Rimini.
        "RO",  # Rovigo.
        "SA",  # Salerno.
        "SI",  # Siena.
        "SO",  # Sondrio.
        "SP",  # La Spezia.
        "SR",  # Siracusa.
        "SS",  # Sassari.
        "SU",  # Sud Sardegna.
        "SV",  # Savona.
        "TA",  # Taranto.
        "TE",  # Teramo.
        "TN",  # Trento.
        "TO",  # Torino.
        "TP",  # Trapani.
        "TR",  # Terni.
        "TS",  # Trieste.
        "TV",  # Treviso.
        "UD",  # Udine.
        "VA",  # Varese.
        "VB",  # Verbano-Cusio-Ossola.
        "VC",  # Vercelli.
        "VE",  # Venezia.
        "VI",  # Vicenza.
        "VR",  # Verona.
        "VT",  # Viterbo.
        "VV",  # Vibo Valentia.
        # Cities.
        "Andria",
        "Barletta",
        "Cesena",
        "Forli",
        "Pesaro",
        "Trani",
        "Urbino",
    )
    subdivisions_aliases = {
        # Provinces.
        "Agrigento": "AG",
        "Alessandria": "AL",
        "Ancona": "AN",
        "Aosta": "AO",
        "Ascoli Piceno": "AP",
        "L'Aquila": "AQ",
        "Arezzo": "AR",
        "Asti": "AT",
        "Avellino": "AV",
        "Bari": "BA",
        "Bergamo": "BG",
        "Biella": "BI",
        "Belluno": "BL",
        "Benevento": "BN",
        "Bologna": "BO",
        "Brindisi": "BR",
        "Brescia": "BS",
        "Barletta-Andria-Trani": "BT",
        "Bolzano": "BZ",
        "Cagliari": "CA",
        "Campobasso": "CB",
        "Caserta": "CE",
        "Chieti": "CH",
        "Caltanissetta": "CL",
        "Cuneo": "CN",
        "Como": "CO",
        "Cremona": "CR",
        "Cosenza": "CS",
        "Catania": "CT",
        "Catanzaro": "CZ",
        "Enna": "EN",
        "Forli-Cesena": "FC",
        "Forlì-Cesena": "FC",
        "Ferrara": "FE",
        "Foggia": "FG",
        "Firenze": "FI",
        "Fermo": "FM",
        "Frosinone": "FR",
        "Genova": "GE",
        "Gorizia": "GO",
        "Grosseto": "GR",
        "Imperia": "IM",
        "Isernia": "IS",
        "Crotone": "KR",
        "Lecco": "LC",
        "Lecce": "LE",
        "Livorno": "LI",
        "Lodi": "LO",
        "Latina": "LT",
        "Lucca": "LU",
        "Monza e Brianza": "MB",
        "Macerata": "MC",
        "Messina": "ME",
        "Milano": "MI",
        "Mantova": "MN",
        "Modena": "MO",
        "Massa-Carrara": "MS",
        "Matera": "MT",
        "Napoli": "NA",
        "Novara": "NO",
        "Nuoro": "NU",
        "Oristano": "OR",
        "Palermo": "PA",
        "Piacenza": "PC",
        "Padova": "PD",
        "Pescara": "PE",
        "Perugia": "PG",
        "Pisa": "PI",
        "Pordenone": "PN",
        "Prato": "PO",
        "Parma": "PR",
        "Pistoia": "PT",
        "Pesaro e Urbino": "PU",
        "Pavia": "PV",
        "Potenza": "PZ",
        "Ravenna": "RA",
        "Reggio Calabria": "RC",
        "Reggio Emilia": "RE",
        "Ragusa": "RG",
        "Rieti": "RI",
        "Roma": "RM",
        "Rimini": "RN",
        "Rovigo": "RO",
        "Salerno": "SA",
        "Siena": "SI",
        "Sondrio": "SO",
        "La Spezia": "SP",
        "Siracusa": "SR",
        "Sassari": "SS",
        "Sud Sardegna": "SU",
        "Savona": "SV",
        "Taranto": "TA",
        "Teramo": "TE",
        "Trento": "TN",
        "Torino": "TO",
        "Trapani": "TP",
        "Terni": "TR",
        "Trieste": "TS",
        "Treviso": "TV",
        "Udine": "UD",
        "Varese": "VA",
        "Verbano-Cusio-Ossola": "VB",
        "Vercelli": "VC",
        "Venezia": "VE",
        "Vicenza": "VI",
        "Verona": "VR",
        "Viterbo": "VT",
        "Vibo Valentia": "VV",
        # Cities.
        "Forlì": "Forli",
    }
    supported_categories = (HALF_DAY, PUBLIC)
    supported_languages = ("en_US", "it_IT", "th")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=ItalyStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Established by Law 1968 of June 23, 1874.
        if self._year >= 1875:
            # New Year's Day.
            self._add_new_years_day(tr("Capodanno"))

        # Established by Royal Decree 5342 of Oct 17, 1869.
        # Abolished by Law 54 of Mar 5, 1977.
        # Restored by Presidential Decree 792 of Dec 28, 1985.
        if self._year <= 1977 or self._year >= 1986:
            # Epiphany.
            self._add_epiphany_day(tr("Epifania"))

        # Established by Law 2765 of Dec 6, 1928.
        # Abolished by Law 54 of Mar 5, 1977.
        if 1929 <= self._year <= 1976:
            # Saint Joseph's Day.
            self._add_saint_josephs_day(tr("San Giuseppe"))

        # Technically established by Royal Decree 5342 of Oct 17, 1869.
        # All and Every Sundays (this includes Easter Sunday) are public holidays.

        # Easter Sunday.
        self._add_easter_sunday(tr("Pasqua"))

        # Established by Law 260 of May 27, 1949.
        if self._year >= 1950:
            # Easter Monday.
            self._add_easter_monday(tr("Lunedì dell'Angelo"))

        # Established by Royal Decree-Law 833 of Apr 19, 1923.
        # Suspended by Royal Decree-Law 781 of Jul 24, 1941.
        if 1923 <= self._year <= 1941:
            # Foundation of Rome.
            self._add_holiday_apr_21(tr("Natale di Roma"))

        # Established by Legislative Decree 185 of Apr 22, 1946.
        # Confirmed by Law 260 of May 27, 1949.
        if self._year >= 1946:
            # Liberation Day.
            self._add_holiday_apr_25(tr("Anniversario della Liberazione"))

            # Labor Day.
            self._add_labor_day(tr("Festa del Lavoro"))

        # Established by Legislative Decree 185 of Apr 22, 1946.
        # Abolished by Law 260 of May 27, 1949.
        if 1946 <= self._year <= 1949:
            # Anniversary of Victory in Europe.
            self._add_world_war_two_victory_day(tr("Anniversario della Vittoria in Europa"))

        # Established by Law 661 of May 5, 1939.
        # Suspended by Royal Decree-Law 781 of Jul 24, 1941.
        if 1939 <= self._year <= 1941:
            # Anniversary of the founding of the Empire.
            self._add_holiday_may_9(tr("Anniversario della fondazione dell'Impero"))

        # 1947 - Legislative Decree 387 of May 28, 1947.
        # 1948 - Decree of May 19, 1948 (?).
        # Established by Law 260 of May 27, 1949.
        # Made movable by Law 54 of Mar 5, 1977.
        # Made fixed by Law 336 of Nov 20, 2000.
        if self._year >= 1947:
            # Republic Day.
            name = tr("Festa della Repubblica")
            if 1977 <= self._year <= 2000:
                self._add_holiday_1st_sun_of_jun(name)
            else:
                self._add_holiday_jun_2(name)

        # Established by Royal Decree 5342 of Oct 17, 1869.
        # Abolished by Law 54 of Mar 5, 1977.
        if self._year <= 1976:
            # Ascension Day.
            self._add_ascension_thursday(tr("Ascensione"))

        # Established by Royal Decree 5342 of Oct 17, 1869.
        # Abolished by Royal Decree 1027 of Aug 4, 1913.
        # Restored by Royal Decree-Law 2859 of Dec 30, 1923.
        # Abolished by Law 54 of Mar 5, 1977.
        if self._year <= 1913 or 1924 <= self._year <= 1976:
            # Saints Peter and Paul.
            self._add_saints_peter_and_paul_day(tr("Santi Apostoli Pietro e Paolo"))

            # Corpus Christi.
            self._add_corpus_christi_day(tr("Corpus Domini"))

        # Established by Royal Decree 5342 of Oct 17, 1869.

        # Assumption Of Mary Day.
        self._add_assumption_of_mary_day(tr("Assunzione della Beata Vergine Maria"))

        # Established by Royal Decree 5342 of Oct 17, 1869.
        # Abolished by Royal Decree 1027 of Aug 4, 1913.
        if self._year <= 1912:
            # Nativity of Mary.
            self._add_nativity_of_mary_day(tr("Natività della Beata Vergine Maria"))

        # Established by Law 401 of Jul 19, 1895.
        # Abolished by Law 1726 of Dec 27, 1930.
        if 1895 <= self._year <= 1930:
            # Anniversary of the capture of Rome.
            self._add_holiday_sep_20(tr("Anniversario della Presa di Roma"))

        # Established by Law 151 of Oct 8, 2025.
        if self._year >= 2026:
            self._add_holiday_oct_4(
                # Saint Francis of Assisi, Patron Saint of Italy.
                tr("Festa nazionale di San Francesco d'Assisi, patrono d'Italia")
            )

        # Established by Royal Decree-Law 1779 of Oct 21, 1926.
        # Date shifted in 1927 by Royal Decree-Law 1922 of Oct 23, 1927.
        # Date changed in 1929 by Royal Decree-Law 1827 of Oct 13, 1929.
        # Suspended by Royal Decree-Law 781 of Jul 24, 1941.
        if 1926 <= self._year <= 1940:
            dates_obs = {
                1927: (OCT, 30),
                1929: (OCT, 27),
            }
            self._add_holiday(
                # Anniversary of the March on Rome.
                tr("Anniversario della Marcia su Roma"),
                dates_obs.get(self._year, (OCT, 28)),
            )

        # Established by Royal Decree 5342 of Oct 17, 1869.

        # All Saints' Day.
        self._add_all_saints_day(tr("Ognissanti"))

        # Established by Royal Decree-Law 1354 of Oct 23, 1922.
        # Date shifted in 1927 by Royal Decree-Law 1922 of Oct 23, 1927.
        # Date changed in 1929 by Royal Decree-Law 1827 of Oct 13, 1929.
        # Suspended by Royal Decree-Law 781 of Jul 24, 1941.
        # Restored by Legislative Decree 185 of Apr 22, 1946.
        # Renamed by Law 260 of May 27, 1949.
        # Made movable by Law 54 of Mar 5, 1977.
        if 1922 <= self._year <= 1940 or self._year >= 1946:
            name = (
                # National Unity Day.
                tr("Giorno dell'unità nazionale")
                if self._year >= 1950
                # Victory Day.
                else tr("Anniversario della Vittoria")
            )
            if self._year >= 1977:
                self._add_holiday_1st_sun_of_nov(name)
            else:
                dates_obs = {
                    1927: (NOV, 6),
                    1929: (NOV, 3),
                }
                self._add_holiday(name, dates_obs.get(self._year, (NOV, 4)))

        # Established by Royal Decree 5342 of Oct 17, 1869.
        # Abolished by Royal Decree 1027 of Aug 4, 1913.
        # Restored by Royal Decree-Law 2859 of Dec 30, 1923.
        if self._year <= 1912 or self._year >= 1924:
            # Immaculate Conception.
            self._add_immaculate_conception_day(tr("Immacolata Concezione"))

        # Established by Royal Decree 5342 of Oct 17, 1869.

        # Christmas Day.
        self._add_christmas_day(tr("Natale"))

        # Established by Law 260 of May 27, 1949.
        if self._year >= 1949:
            # Saint Stephen's Day.
            self._add_christmas_day_two(tr("Santo Stefano"))

    def _populate_subdiv_holidays(self):
        # The municipal festivals of the patron saint (Festa del santo patrono).
        # Established by Royal Decree 5342 of Oct 17, 1869.
        # Abolished by Royal Decree 1027 of Aug 4, 1913.
        # Restored by Law 260 of May 27, 1949.
        if self._year <= 1913 or self._year >= 1950:
            super()._populate_subdiv_holidays()

    def _populate_subdiv_ag_public_holidays(self):
        # Saint Gerland's Day.
        self._add_holiday_feb_25(tr("San Gerlando"))

    def _populate_subdiv_al_public_holidays(self):
        # Saint Baudolino's Day.
        self._add_holiday_nov_10(tr("San Baudolino"))

    def _populate_subdiv_an_public_holidays(self):
        # Saint Cyriacus's Day.
        self._add_holiday_may_4(tr("San Ciriaco"))

    def _populate_subdiv_ao_public_holidays(self):
        # Saint Grat's Day.
        self._add_holiday_sep_7(tr("San Grato"))

    def _populate_subdiv_ap_public_holidays(self):
        # Saint Emidius's Day.
        self._add_holiday_aug_5(tr("Sant'Emidio"))

    def _populate_subdiv_aq_public_holidays(self):
        # Saint Maximus of Aveia's Day.
        self._add_holiday_jun_10(tr("San Massimo d'Aveia"))

    def _populate_subdiv_ar_public_holidays(self):
        # Saint Donatus of Arezzo's Day.
        self._add_holiday_aug_7(tr("San Donato d'Arezzo"))

    def _populate_subdiv_at_public_holidays(self):
        # Saint Secundus of Asti's Day.
        self._add_holiday_1st_tue_of_may(tr("San Secondo di Asti"))

    def _populate_subdiv_av_public_holidays(self):
        # Saint Modestinus's Day.
        self._add_holiday_feb_14(tr("San Modestino"))

    def _populate_subdiv_ba_public_holidays(self):
        # Saint Nicholas's Day.
        self._add_holiday_dec_6(tr("San Nicola"))

    def _populate_subdiv_bg_public_holidays(self):
        # Saint Alexander of Bergamo's Day.
        self._add_holiday_aug_26(tr("Sant'Alessandro di Bergamo"))

    def _populate_subdiv_bi_public_holidays(self):
        # Saint Stephen's Day.
        self._add_christmas_day_two(tr("Santo Stefano"))

    def _populate_subdiv_bl_public_holidays(self):
        # Saint Martin's Day.
        self._add_saint_martins_day(tr("San Martino"))

    def _populate_subdiv_bn_public_holidays(self):
        # Saint Bartholomew's Day.
        self._add_holiday_aug_24(tr("San Bartolomeo apostolo"))

    def _populate_subdiv_bo_public_holidays(self):
        # Saint Petronius's Day.
        self._add_holiday_oct_4(tr("San Petronio"))

    def _populate_subdiv_br_public_holidays(self):
        # Saint Theodore of Amasea and Saint Lawrence of Brindisi's Day.
        self._add_holiday_1st_sun_of_sep(tr("San Teodoro d'Amasea e San Lorenzo da Brindisi"))

    def _populate_subdiv_bs_public_holidays(self):
        # Saints Faustinus and Jovita's Day.
        self._add_holiday_feb_15(tr("Santi Faustino e Giovita"))

    def _populate_subdiv_bt_public_holidays(self):
        # Trani province holiday.

        # Saint Nicholas the Pilgrim's Day.
        self._add_holiday_may_3(tr("San Nicola Pellegrino"))

        # Andria province holiday.

        # Saint Richard of Andria's Day.
        self._add_holiday_3rd_sun_of_sep(tr("San Riccardo di Andria"))

        # Barletta province holiday.

        # Saint Roger's Day.
        self._add_holiday_dec_30(tr("San Ruggero"))

    def _populate_subdiv_bz_public_holidays(self):
        # Province Law 36 of Oct 16, 1992.
        if self._year >= 1993:
            # Whit Monday.
            self._add_whit_monday(tr("Lunedì di Pentecoste"))

    def _populate_subdiv_bz_half_day_holidays(self):
        # Province Law 36 of Oct 16, 1992.
        if self._year >= 1993:
            # Fat Thursday.
            self._add_holiday_52_days_prior_easter(tr("Giovedì grasso"))

            # Last Day of Carnival.
            self._add_carnival_tuesday(tr("Ultimo giorno di carnevale"))

            # Good Friday.
            self._add_good_friday(tr("Venerdì santo"))

        if self._year >= 1992:
            # Christmas Eve.
            self._add_christmas_eve(tr("Vigilia di Natale"))

            # Last Day of the Year.
            self._add_new_years_eve(tr("Ultimo giorno dell'anno"))

    def _populate_subdiv_ca_public_holidays(self):
        # Saint Saturninus of Cagliari's Day.
        self._add_holiday_oct_30(tr("San Saturnino di Cagliari"))

    def _populate_subdiv_cb_public_holidays(self):
        # Saint George's Day.
        self._add_saint_georges_day(tr("San Giorgio"))

    def _populate_subdiv_ce_public_holidays(self):
        # Saint Sebastian's Day.
        self._add_holiday_jan_20(tr("San Sebastiano"))

    def _populate_subdiv_ch_public_holidays(self):
        # Saint Justin of Chieti's Day.
        self._add_holiday_may_11(tr("San Giustino di Chieti"))

    def _populate_subdiv_cl_public_holidays(self):
        # Saint Michael the Archangel's Day.
        self._add_holiday_sep_29(tr("San Michele Arcangelo"))

    def _populate_subdiv_cn_public_holidays(self):
        # Saint Michael the Archangel's Day.
        self._add_holiday_sep_29(tr("San Michele Arcangelo"))

    def _populate_subdiv_co_public_holidays(self):
        # Saint Abbondius's Day.
        self._add_holiday_aug_31(tr("Sant'Abbondio"))

    def _populate_subdiv_cr_public_holidays(self):
        # Saint Homobonus's Day.
        self._add_holiday_nov_13(tr("Sant'Omobono"))

    def _populate_subdiv_cs_public_holidays(self):
        # Our Lady of the Pilerio.
        self._add_holiday_feb_12(tr("Madonna del Pilerio"))

    def _populate_subdiv_ct_public_holidays(self):
        # Saint Agatha's Day.
        self._add_holiday_feb_5(tr("Sant'Agata"))

    def _populate_subdiv_cz_public_holidays(self):
        # Saint Vitalian's Day.
        self._add_holiday_jul_16(tr("San Vitaliano"))

    def _populate_subdiv_en_public_holidays(self):
        # Visitation of Mary Day.
        self._add_holiday_jul_2(tr("Madonna della Visitazione"))

    def _populate_subdiv_fc_public_holidays(self):
        # Forlì province holiday.

        # Our Lady of the Fire.
        self._add_holiday_feb_4(tr("Madonna del Fuoco"))

        # Cesena province holiday.

        # Saint John's Day.
        self._add_saint_johns_day(tr("San Giovanni Battista"))

    def _populate_subdiv_fe_public_holidays(self):
        # Saint George's Day.
        self._add_saint_georges_day(tr("San Giorgio"))

    def _populate_subdiv_fg_public_holidays(self):
        # Our Lady of the Seven Veils.
        self._add_holiday_mar_22(tr("Madonna dei Sette Veli"))

    def _populate_subdiv_fi_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("San Giovanni Battista"))

    def _populate_subdiv_fm_public_holidays(self):
        # Assumption Day.
        name = tr("Maria Santissima Assunta")
        self._add_assumption_of_mary_day(name)
        self._add_holiday_aug_16(name)

    def _populate_subdiv_fr_public_holidays(self):
        # Saint Silverius's Day.
        self._add_holiday_jun_20(tr("San Silverio"))

    def _populate_subdiv_ge_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("San Giovanni Battista"))

    def _populate_subdiv_go_public_holidays(self):
        # Saints Hilary and Tatian's Day.
        self._add_holiday_mar_16(tr("Santi Ilario e Taziano"))

    def _populate_subdiv_gr_public_holidays(self):
        # Saint Lawrence's Day.
        self._add_holiday_aug_10(tr("San Lorenzo"))

    def _populate_subdiv_im_public_holidays(self):
        # Saint Leonard of Porto Maurizio's Day.
        self._add_holiday_nov_26(tr("San Leonardo da Porto Maurizio"))

    def _populate_subdiv_is_public_holidays(self):
        # Saint Peter Celestine's Day.
        self._add_holiday_may_19(tr("San Pietro Celestino"))

    def _populate_subdiv_kr_public_holidays(self):
        # Saint Dionysius's Day.
        self._add_holiday_oct_9(tr("San Dionigi"))

    def _populate_subdiv_lc_public_holidays(self):
        # Saint Nicholas's Day.
        self._add_holiday_dec_6(tr("San Nicola"))

    def _populate_subdiv_le_public_holidays(self):
        # Saint Orontius's Day.
        self._add_holiday_aug_26(tr("Sant'Oronzo"))

    def _populate_subdiv_li_public_holidays(self):
        # Saint Julia's Day.
        self._add_holiday_may_22(tr("Santa Giulia"))

    def _populate_subdiv_lo_public_holidays(self):
        # Saint Bassianus's Day.
        self._add_holiday_jan_19(tr("San Bassiano"))

    def _populate_subdiv_lt_public_holidays(self):
        # Saint Mark's Day.
        self._add_holiday_apr_25(tr("San Marco Evangelista"))

        # Saint Maria Goretti's Day.
        self._add_holiday_jul_6(tr("Santa Maria Goretti"))

    def _populate_subdiv_lu_public_holidays(self):
        # Saint Paulinus of Lucca's Day.
        self._add_holiday_jul_12(tr("San Paolino di Lucca"))

    def _populate_subdiv_mb_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("San Giovanni Battista"))

    def _populate_subdiv_mc_public_holidays(self):
        # Saint Julian the Hospitaller's Day.
        self._add_holiday_aug_31(tr("San Giuliano l'ospitaliere"))

    def _populate_subdiv_me_public_holidays(self):
        # Our Lady of the Letter.
        self._add_holiday_jun_3(tr("Madonna della Lettera"))

    def _populate_subdiv_mi_public_holidays(self):
        # Saint Ambrose's Day.
        self._add_holiday_dec_7(tr("Sant'Ambrogio"))

    def _populate_subdiv_mn_public_holidays(self):
        # Saint Anselm of Baggio's Day.
        self._add_holiday_mar_18(tr("Sant'Anselmo da Baggio"))

    def _populate_subdiv_mo_public_holidays(self):
        # Saint Geminianus's Day.
        self._add_holiday_jan_31(tr("San Geminiano"))

    def _populate_subdiv_ms_public_holidays(self):
        # Saint Francis of Assisi's Day.
        self._add_holiday_oct_4(tr("San Francesco d'Assisi"))

    def _populate_subdiv_mt_public_holidays(self):
        # Our Lady of the Bruna.
        self._add_holiday_jul_2(tr("Madonna della Bruna"))

    def _populate_subdiv_na_public_holidays(self):
        # Saint Januarius's Day.
        self._add_holiday_sep_19(tr("San Gennaro"))

    def _populate_subdiv_no_public_holidays(self):
        # Saint Gaudentius's Day.
        self._add_holiday_jan_22(tr("San Gaudenzio"))

    def _populate_subdiv_nu_public_holidays(self):
        # Our Lady of the Snows.
        self._add_holiday_aug_5(tr("Nostra Signora della Neve"))

    def _populate_subdiv_or_public_holidays(self):
        # Saint Archelaus's Day.
        self._add_holiday_feb_13(tr("Sant'Archelao"))

    def _populate_subdiv_pa_public_holidays(self):
        # Saint Rosalia's Day.
        self._add_holiday_jul_15(tr("Santa Rosalia"))

    def _populate_subdiv_pc_public_holidays(self):
        # Saint Antoninus of Piacenza's Day.
        self._add_holiday_jul_4(tr("Sant'Antonino di Piacenza"))

    def _populate_subdiv_pd_public_holidays(self):
        # Saint Anthony of Padua's Day.
        self._add_saint_anthonys_day(tr("Sant'Antonio di Padova"))

    def _populate_subdiv_pe_public_holidays(self):
        # Saint Cetteus's Day.
        self._add_holiday_oct_10(tr("San Cetteo"))

    def _populate_subdiv_pg_public_holidays(self):
        # Saint Clare of Assisi's Day.
        self._add_holiday_aug_11(tr("Santa Chiara d'Assisi"))

        # Saint Francis of Assisi's Day.
        self._add_holiday_oct_4(tr("San Francesco d'Assisi"))

    def _populate_subdiv_pi_public_holidays(self):
        # Saint Ranieri's Day.
        self._add_holiday_jun_17(tr("San Ranieri"))

    def _populate_subdiv_pn_public_holidays(self):
        # Saint Mark's Day.
        self._add_holiday_apr_25(tr("San Marco Evangelista"))

        # Our Lady of Graces.
        self._add_nativity_of_mary_day(tr("Madonna delle Grazie"))

    def _populate_subdiv_po_public_holidays(self):
        # Saint Stephen's Day.
        self._add_christmas_day_two(tr("Santo Stefano"))

    def _populate_subdiv_pr_public_holidays(self):
        # Saint Hilary of Poitiers's Day.
        self._add_holiday_jan_13(tr("Sant'Ilario di Poitiers"))

    def _populate_subdiv_pt_public_holidays(self):
        # Saint James's Day.
        self._add_saint_james_day(tr("San Jacopo"))

    def _populate_subdiv_pu_public_holidays(self):
        # Urbino province holiday.

        # Saint Crescentinus's Day.
        self._add_holiday_jun_1(tr("San Crescentino"))

        # Pesaro province holiday.

        # Saint Terentius of Pesaro's Day.
        self._add_holiday_sep_24(tr("San Terenzio di Pesaro"))

    def _populate_subdiv_pv_public_holidays(self):
        # Saint Syrus's Day.
        self._add_holiday_dec_9(tr("San Siro"))

    def _populate_subdiv_pz_public_holidays(self):
        # Saint Gerard of Potenza's Day.
        self._add_holiday_may_30(tr("San Gerardo di Potenza"))

    def _populate_subdiv_ra_public_holidays(self):
        # Saint Apollinaris's Day.
        self._add_holiday_jul_23(tr("Sant'Apollinare"))

    def _populate_subdiv_rc_public_holidays(self):
        # Saint George's Day.
        self._add_saint_georges_day(tr("San Giorgio"))

    def _populate_subdiv_re_public_holidays(self):
        # Saint Prosper of Reggio's Day.
        self._add_holiday_nov_24(tr("San Prospero Vescovo"))

    def _populate_subdiv_rg_public_holidays(self):
        # Saint George's Day.
        self._add_saint_georges_day(tr("San Giorgio Martire"))

        # Saint John's Day.
        self._add_holiday_aug_29(tr("San Giovanni Battista"))

    def _populate_subdiv_ri_public_holidays(self):
        # Saint Barbara's Day.
        self._add_holiday_dec_4(tr("Santa Barbara"))

    def _populate_subdiv_rm_public_holidays(self):
        # Saints Peter and Paul's Day.
        self._add_saints_peter_and_paul_day(tr("Santi Pietro e Paolo"))

    def _populate_subdiv_rn_public_holidays(self):
        # Saint Gaudentius's Day.
        self._add_holiday_oct_14(tr("San Gaudenzio"))

    def _populate_subdiv_ro_public_holidays(self):
        # Saint Bellinus's Day.
        self._add_holiday_nov_26(tr("San Bellino"))

    def _populate_subdiv_sa_public_holidays(self):
        # Saint Matthew's Day.
        self._add_holiday_sep_21(tr("San Matteo Evangelista"))

    def _populate_subdiv_si_public_holidays(self):
        # Saint Ansanus's Day.
        self._add_holiday_dec_1(tr("Sant'Ansano"))

    def _populate_subdiv_so_public_holidays(self):
        # Saints Gervasius and Protasius's Day.
        self._add_holiday_jun_19(tr("San Gervasio e San Protasio"))

    def _populate_subdiv_sp_public_holidays(self):
        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("San Giuseppe"))

    def _populate_subdiv_sr_public_holidays(self):
        # Saint Lucy's Day.
        self._add_holiday_dec_13(tr("Santa Lucia"))

    def _populate_subdiv_ss_public_holidays(self):
        # Saint Nicholas's Day.
        self._add_holiday_dec_6(tr("San Nicola"))

    def _populate_subdiv_su_public_holidays(self):
        # Carbonia province holiday.

        # Saint Pontian's Day.
        self._add_holiday_4_days_past_2nd_sun_of_may(tr("San Ponziano"))

    def _populate_subdiv_sv_public_holidays(self):
        # Our Lady of Mercy.
        self._add_holiday_mar_18(tr("Nostra Signora della Misericordia"))

    def _populate_subdiv_ta_public_holidays(self):
        # Saint Catald's Day.
        self._add_holiday_may_10(tr("San Cataldo"))

    def _populate_subdiv_te_public_holidays(self):
        # Saint Berardo's Day.
        self._add_holiday_dec_19(tr("San Berardo da Pagliara"))

    def _populate_subdiv_tn_public_holidays(self):
        # Saint Vigilius's Day.
        self._add_holiday_jun_26(tr("San Vigilio"))

    def _populate_subdiv_to_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("San Giovanni Battista"))

    def _populate_subdiv_tp_public_holidays(self):
        # Saint Albert of Trapani's Day.
        self._add_holiday_aug_7(tr("Sant'Alberto degli Abati"))

    def _populate_subdiv_tr_public_holidays(self):
        # Saint Valentine's Day.
        self._add_holiday_feb_14(tr("San Valentino"))

    def _populate_subdiv_ts_public_holidays(self):
        # Saint Justus's Day.
        self._add_holiday_nov_3(tr("San Giusto"))

    def _populate_subdiv_tv_public_holidays(self):
        # Saint Liberal's Day.
        self._add_holiday_apr_27(tr("San Liberale"))

    def _populate_subdiv_ud_public_holidays(self):
        # Saints Hermagoras and Fortunatus's Day.
        self._add_holiday_jul_12(tr("Santi Ermacora e Fortunato"))

    def _populate_subdiv_va_public_holidays(self):
        # Saint Victor the Moor's Day.
        self._add_holiday_may_8(tr("San Vittore il Moro"))

    def _populate_subdiv_vb_public_holidays(self):
        # Verbania province holiday.

        # Saint Victor the Moor's Day.
        self._add_holiday_may_8(tr("San Vittore il Moro"))

    def _populate_subdiv_vc_public_holidays(self):
        # Saint Eusebius of Vercelli's Day.
        self._add_holiday_aug_1(tr("Sant'Eusebio di Vercelli"))

    def _populate_subdiv_ve_public_holidays(self):
        # Saint Mark's Day.
        self._add_holiday_apr_25(tr("San Marco Evangelista"))

        # Our Lady of Health.
        self._add_holiday_nov_21(tr("Madonna della Salute"))

    def _populate_subdiv_vi_public_holidays(self):
        # Our Lady of Monte Berico.
        self._add_nativity_of_mary_day(tr("Madonna di Monte Berico"))

    def _populate_subdiv_vr_public_holidays(self):
        # Saint Zeno's Day.
        self._add_holiday_may_21(tr("San Zeno"))

    def _populate_subdiv_vt_public_holidays(self):
        # Saint Rose of Viterbo's Day.
        self._add_holiday_sep_4(tr("Santa Rosa da Viterbo"))

    def _populate_subdiv_vv_public_holidays(self):
        # Saint Leoluca's Day.
        self._add_holiday_mar_1(tr("San Leoluca"))

    def _populate_subdiv_andria_public_holidays(self):
        # Saint Richard of Andria's Day.
        self._add_holiday_3rd_sun_of_sep(tr("San Riccardo di Andria"))

    def _populate_subdiv_barletta_public_holidays(self):
        # Saint Roger's Day.
        self._add_holiday_dec_30(tr("San Ruggero"))

    def _populate_subdiv_cesena_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("San Giovanni Battista"))

    def _populate_subdiv_forli_public_holidays(self):
        # Our Lady of the Fire.
        self._add_holiday_feb_4(tr("Madonna del Fuoco"))

    def _populate_subdiv_pesaro_public_holidays(self):
        # Saint Terentius of Pesaro's Day.
        self._add_holiday_sep_24(tr("San Terenzio di Pesaro"))

    def _populate_subdiv_trani_public_holidays(self):
        # Saint Nicholas the Pilgrim's Day.
        self._add_holiday_may_3(tr("San Nicola Pellegrino"))

    def _populate_subdiv_urbino_public_holidays(self):
        # Saint Crescentinus's Day.
        self._add_holiday_jun_1(tr("San Crescentino"))


class IT(Italy):
    pass


class ITA(Italy):
    pass


class ItalyStaticHolidays:
    """Italy special holidays.

    References:
        * [Law 366 of Jun 28, 1907](https://web.archive.org/web/20251124204309/https://www.gazzettaufficiale.it/eli/gu/1907/06/28/152/sg/pdf)
        * [Law 450 of Jul 7, 1910](https://web.archive.org/web/20251027060106/https://www.gazzettaufficiale.it/eli/gu/1910/07/18/167/sg/pdf)
        * [Royal Decree 269 of Mar 11, 1920](https://web.archive.org/web/20251027093344/https://www.gazzettaufficiale.it/eli/gu/1920/03/19/66/sg/pdf)
        * [Royal Decree 1208 of Aug 21, 1921](https://web.archive.org/web/20260117043612/https://www.gazzettaufficiale.it/eli/gu/1921/09/10/215/sg/pdf)
        * [Royal Decree 1207 of Jul 10, 1925](https://web.archive.org/web/20251027092027/https://www.gazzettaufficiale.it/eli/gu/1925/07/24/170/sg/pdf)
        * [Royal Decree-Law 376 of Apr 25, 1938](https://web.archive.org/web/20251027093701/https://www.gazzettaufficiale.it/eli/gu/1938/04/28/97/sg/pdf)
        * [Presidential Legislative Decree 2 of Jun 19, 1946](https://web.archive.org/web/20251213235623/https://www.normattiva.it/atto/caricaDettaglioAtto?atto.dataPubblicazioneGazzetta=1946-06-20&atto.codiceRedazionale=046U0002&tipoDettaglio=originario&qId=0c1f85e7-d249-4987-966d-d9a4073ef48f)
    """

    # Anniversary of the Unification of Italy.
    anniversary_of_unification = tr("Anniversario dell'Unità d'Italia")

    special_public_holidays = {
        # 100th anniversary of the birth of General Giuseppe Garibaldi.
        1907: (JUL, 4, tr("Centenario della nascita del generale Giuseppe Garibaldi")),
        # 100th anniversary of the birth of Camillo Cavour.
        1910: (AUG, 10, tr("Centenario della nascita di Camillo Cavour")),
        1911: (MAR, 17, anniversary_of_unification),
        # 100th anniversary of the birth of Victor Emmanuel II.
        1920: (MAR, 14, tr("Centenario della nascita del Vittorio Emanuele II")),
        # Celebration of 600th anniversary of the death of Dante.
        1921: (SEP, 14, tr("Celebrazione del sesto centenario dantesco")),
        1926: (
            OCT,
            4,
            # 700th anniversary of the death of Saint Francis of Assisi.
            tr("Anniversario del VII centenario della morte di San Francesco di Assisi"),
        ),
        # National Holiday.
        1938: (MAY, 3, tr("Festa Nazionale")),
        # Public Holiday.
        1946: (JUN, 11, tr("Giorno Festivo")),
        1961: (MAR, 17, anniversary_of_unification),
        2011: (MAR, 17, anniversary_of_unification),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.switzerland import Switzerland
from tests.common import CommonCountryTests


class TestSwitzerland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(1970, 2050)
        super().setUpClass(
            Switzerland,
            with_subdiv_categories=True,
            years_subdiv_gl=range(1834, 2050),
            years_subdiv_stadt_zurich=range(1898, 2050),
        )

    def test_all_holidays_present(self):
        y_2018 = set()
        for p in Switzerland.subdivisions:
            y_2018.update(
                Switzerland(
                    categories=Switzerland.supported_categories, years=2018, subdiv=p
                ).values()
            )
        all_h = {  # Holidays names in their chronological order.
            "Neujahrstag",
            "Berchtoldstag",
            "Heilige Drei Könige",
            "Jahrestag der Ausrufung der Republik",
            "Josefstag",
            "Näfelser Fahrt",
            "Vortag vor Karfreitag",
            "Karfreitag",
            "Ostersonntag",
            "Ostermontag",
            "Sechseläuten",
            "Tag der Arbeit",
            "Vortag vor Auffahrt",
            "Auffahrt",
            "Brückentag nach Auffahrt",
            "Pfingstsonntag",
            "Pfingstmontag",
            "Fronleichnam",
            "Fest der Unabhängigkeit",
            "Peter und Paul",
            "Nationalfeiertag",
            "Mariä Himmelfahrt",
            "Knabenschiessen",
            "Eidgenössischer Bettag",
            "Bettagsmontag",
            "Bruder Klaus",
            "Allerheiligen",
            "Mariä Empfängnis",
            "Genfer Bettag",
            "Heiligabend",
            "Weihnachten",
            "Stephanstag",
            "Brückentag",
            "Vortag vor Neujahr",
            "Wiederherstellung der Republik",
        }

        self.assertEqual(
            all_h,
            y_2018,
            f"missing: {all_h - y_2018 or 'no'}, extra: {y_2018 - all_h or 'no'}",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Neujahrstag", (f"{year}-01-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Auffahrt"
        dts = (
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.full_range)
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, dts)

    def test_national_day(self):
        self.assertHolidayName("Nationalfeiertag", (f"{year}-08-01" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Weihnachten", (f"{year}-12-25" for year in self.full_range))

    def test_berchtolds_day(self):
        name = "Berchtoldstag"
        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AG", "BE", "JU", "LU", "TG", "VD", "Stadt Zurich"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-02" for year in self.full_range)
                )
            elif subdiv != "NE":
                self.assertNoHoliday(holidays, (f"{year}-01-02" for year in self.full_range))
                self.assertNoHolidayName(name, holidays)
        # OPTIONAL.
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv in {"FR", "GL", "NW", "OW", "SG", "SH", "SO", "VS", "ZG"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-02" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

        # When holiday present in NE.
        ne_years = {1978, 1984, 1989, 1995, 2006, 2012, 2017, 2023, 2034, 2040, 2045}
        self.assertSubdivNeHolidayName(name, (f"{year}-01-02" for year in ne_years))
        self.assertNoSubdivNeHoliday(
            f"{year}-01-02" for year in self.full_range if year not in ne_years
        )

    def test_epiphany(self):
        name = "Heilige Drei Könige"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"SZ", "TI", "UR"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-06" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_republic_day(self):
        name = "Jahrestag der Ausrufung der Republik"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NE":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_st_josephs_day(self):
        name = "Josefstag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"NW", "SZ", "TI", "UR", "VS"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-19" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_nafels_ride(self):
        name = "Näfelser Fahrt"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "GL":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-02",
                    "2021-04-08",
                    "2022-04-07",
                    "2023-04-13",
                    "2024-04-04",
                    "2025-04-03",
                )
                self.assertHolidayName(name, holidays, range(1835, self.end_year))
                self.assertNoHolidayName(name, holidays, 1834)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_good_friday(self):
        name = "Karfreitag"
        dts = (
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        subdivs_that_dont = {"TI", "VS"}
        subdivs_optional = {"GR"}
        subdivs_that_have = set(Switzerland.subdivisions) - subdivs_that_dont - subdivs_optional

        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in subdivs_that_have:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)
        # OPTIONAL.
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv in subdivs_optional:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_easter_sunday(self):
        name = "Ostersonntag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_de_facto_holidays.items():
            if subdiv in {"ZH", "Stadt Zurich"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-12",
                    "2021-04-04",
                    "2022-04-17",
                    "2023-04-09",
                    "2024-03-31",
                    "2025-04-20",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_easter_monday(self):
        name = "Ostermontag"
        dts = (
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        subdivs_that_dont = {"NE", "SO"}
        subdivs_optional = {"FR", "NW", "OW", "VS", "ZG"}
        subdivs_that_have = set(Switzerland.subdivisions) - subdivs_that_dont - subdivs_optional

        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in subdivs_that_have:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)
        # OPTIONAL.
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv in subdivs_optional:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_labor_day(self):
        name = "Tag der Arbeit"
        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AG", "BL", "BS", "JU", "NE", "SH", "TG", "TI", "ZH", "Stadt Zurich"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)
        # HALF-DAY.
        for subdiv, holidays in self.subdiv_half_day_holidays.items():
            if subdiv == "SO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_whit_sunday(self):
        name = "Pfingstsonntag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_de_facto_holidays.items():
            if subdiv in {"ZH", "Stadt Zurich"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-05-31",
                    "2021-05-23",
                    "2022-06-05",
                    "2023-05-28",
                    "2024-05-19",
                    "2025-06-08",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_whit_monday(self):
        name = "Pfingstmontag"
        dts = (
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        subdivs_that_dont = {"NE", "SO"}
        subdivs_optional = {"FR", "NW", "OW", "VS", "ZG"}
        subdivs_that_have = set(Switzerland.subdivisions) - subdivs_that_dont - subdivs_optional

        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in subdivs_that_have:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            elif subdiv in subdivs_that_dont:
                self.assertNoHolidayName(name, holidays)
        # OPTIONAL.
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv in subdivs_optional:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_corpus_christi(self):
        name = "Fronleichnam"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AG", "AI", "JU", "LU", "NE", "NW", "OW", "SZ", "TI", "UR", "VS", "ZG"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-06-11",
                    "2021-06-03",
                    "2022-06-16",
                    "2023-06-08",
                    "2024-05-30",
                    "2025-06-19",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_independence_day(self):
        name = "Fest der Unabhängigkeit"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "JU":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-23" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saints_peter_and_paul(self):
        name = "Peter und Paul"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-29" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_assumption_day(self):
        name = "Mariä Himmelfahrt"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AG", "AI", "JU", "LU", "NW", "OW", "SZ", "TI", "UR", "VS", "ZG"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-15" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_federal_day_of_thanksgiving_prayer_and_repentance(self):
        name = "Eidgenössischer Bettag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_de_facto_holidays.items():
            if subdiv in {"ZH", "Stadt Zurich"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-20",
                    "2021-09-19",
                    "2022-09-18",
                    "2023-09-17",
                    "2024-09-15",
                    "2025-09-21",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_prayer_monday(self):
        name = "Bettagsmontag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "VD":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-21",
                    "2021-09-20",
                    "2022-09-19",
                    "2023-09-18",
                    "2024-09-16",
                    "2025-09-22",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_nicholas(self):
        name = "Bruder Klaus"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "OW":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-25" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_all_saints_day(self):
        name = "Allerheiligen"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {
                "AG",
                "AI",
                "GL",
                "JU",
                "LU",
                "NW",
                "OW",
                "SG",
                "SZ",
                "TI",
                "UR",
                "VS",
                "ZG",
            }:
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_genevan_fast(self):
        name = "Genfer Bettag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "GE":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-10",
                    "2021-09-09",
                    "2022-09-08",
                    "2023-09-07",
                    "2024-09-05",
                    "2025-09-11",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_immaculate_conception(self):
        name = "Mariä Empfängnis"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AG", "AI", "LU", "NW", "OW", "SZ", "TI", "UR", "VS", "ZG"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-08" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_stephens_day(self):
        name = "Stephanstag"
        subdivs_that_have = {
            "AG",
            "BL",
            "BS",
            "BE",
            "GL",
            "GR",
            "LU",
            "SG",
            "SH",
            "SZ",
            "TG",
            "TI",
            "ZH",
            "Stadt Zurich",
        }
        subdivs_optional = {"FR", "NW", "OW", "VS", "ZG"}
        subdivs_that_dont = (
            set(Switzerland.subdivisions)
            - subdivs_that_have
            - subdivs_optional
            - {"AI", "AR", "NE", "UR"}
        )

        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in subdivs_that_have:
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-26" for year in self.full_range)
                )
            elif subdiv in subdivs_that_dont:
                self.assertNoHolidayName(name, holidays)
        # OPTIONAL.
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv in subdivs_optional:
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-26" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

        # When holiday present in NE.
        ne_years_have = {1977, 1983, 1988, 1994, 2005, 2011, 2016, 2022, 2033, 2039, 2044}
        self.assertSubdivNeHolidayName(name, (f"{year}-12-26" for year in ne_years_have))
        self.assertNoSubdivNeHoliday(
            f"{year}-12-26" for year in self.full_range if year not in ne_years_have
        )

        # When holiday not present in AI, AR, UR.
        ai_ar_ur_years_dont = {
            1970,
            1972,
            1978,
            1981,
            1987,
            1989,
            1992,
            1995,
            1998,
            2000,
            2006,
            2009,
            2015,
            2017,
            2020,
            2023,
            2026,
            2028,
            2034,
            2037,
            2043,
            2045,
            2048,
        }
        for subdiv in ("AI", "AR", "UR"):
            self.assertHolidayName(
                name,
                self.subdiv_holidays[subdiv],
                (year for year in self.full_range if year not in ai_ar_ur_years_dont),
            )
            self.assertNoHoliday(
                self.subdiv_holidays[subdiv], (f"{year}-12-26" for year in ai_ar_ur_years_dont)
            )

    def test_restoration_day(self):
        name = "Wiederherstellung der Republik"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "GE":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-31" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_knabenschiessen(self):
        name = "Knabenschiessen"
        self.assertNoHolidayName(name)
        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-12",
                    "2020-09-13",
                    "2021-09-11",
                    "2021-09-12",
                    "2022-09-10",
                    "2022-09-11",
                    "2023-09-09",
                    "2023-09-10",
                    "2024-09-07",
                    "2024-09-08",
                    "2025-09-13",
                    "2025-09-14",
                )
                self.assertHolidayNameCount(name, 2, holidays, range(1899, self.end_year))
                self.assertNoHolidayName(name, holidays, 1898)
            else:
                self.assertNoHolidayName(name, holidays)
        # HALF-DAY.
        for subdiv, holidays in self.subdiv_half_day_holidays.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-14",
                    "2021-09-13",
                    "2022-09-12",
                    "2023-09-11",
                    "2024-09-09",
                    "2025-09-15",
                )
                self.assertHolidayName(name, holidays, range(1899, self.end_year))
                self.assertNoHolidayName(name, holidays, 1898)
            else:
                self.assertNoHolidayName(name, holidays)
        # OPTIONAL.
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-14",
                    "2023-09-11",
                    "2024-09-09",
                    "2025-09-15",
                )
                self.assertNoHolidayName(
                    name,
                    holidays,
                    # Not in years where 4 bridge holidays are given out for Christmas-New Year.
                    "2016-09-12",
                    "2021-09-13",
                    "2022-09-12",
                    "2027-09-13",
                )
                self.assertNoHolidayName(name, holidays, 1898)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_before_good_friday(self):
        name = "Vortag vor Karfreitag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in {
            **self.subdiv_half_day_holidays,
            **self.subdiv_optional_holidays,
        }.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-09",
                    "2021-04-01",
                    "2022-04-14",
                    "2023-04-06",
                    "2024-03-28",
                    "2025-04-17",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sechselauten(self):
        name = "Sechseläuten"
        self.assertNoHolidayName(name)
        for subdiv, holidays in {
            **self.subdiv_half_day_holidays,
            **self.subdiv_optional_holidays,
        }.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    # 1st Mon after Vernal Equinox.
                    "1902-03-24",
                    "1903-03-23",
                    "1904-03-21",
                    "1949-03-21",
                    "1950-03-27",
                    "1951-03-26",
                    # 3rd Mon of April, 4th if Holy week.
                    "1952-04-21",
                    "2020-04-20",
                    "2021-04-19",
                    "2022-04-25",
                    "2023-04-17",
                    "2024-04-15",
                    "2025-04-28",
                )
                self.assertHolidayName(name, holidays, range(1902, self.end_year))
                self.assertNoHolidayName(name, holidays, 1901)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_before_ascension_day(self):
        name = "Vortag vor Auffahrt"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_half_day_holidays.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-05-20",
                    "2021-05-12",
                    "2022-05-25",
                    "2023-05-17",
                    "2024-05-08",
                    "2025-05-28",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_bridge_holiday_for_ascension_day(self):
        name = "Brückentag nach Auffahrt"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_optional_holidays.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2018-05-11",
                    "2019-05-31",
                    "2024-05-10",
                    "2025-05-30",
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_christmas_eve(self):
        name = "Heiligabend"
        self.assertNoHolidayName(name)
        for subdiv, holidays in {
            **self.subdiv_half_day_holidays,
            **self.subdiv_optional_holidays,
        }.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-24" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_bridge_holidays(self):
        name = "Brückentag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_optional_holidays_non_observed.items():
            if subdiv == "Stadt Zurich":
                self.assertNonObservedHolidayName(
                    name,
                    holidays,
                    (f"{year}-12-27" for year in self.full_range),
                    (f"{year}-12-28" for year in self.full_range),
                    (f"{year}-12-29" for year in self.full_range),
                    (f"{year}-12-30" for year in self.full_range),
                )
            else:
                self.assertNoNonObservedHolidayName(name, holidays)

        self.assertNoSubdivStadtZurichOptionalHoliday(
            "2025-12-27",
            "2025-12-28",
            "2026-12-27",
        )

    def test_new_years_eve(self):
        name = "Vortag vor Neujahr"
        self.assertNoHolidayName(name)
        for subdiv, holidays in {
            **self.subdiv_half_day_holidays,
            **self.subdiv_optional_holidays,
        }.items():
            if subdiv == "Stadt Zurich":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-31" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Neujahrstag"),
            ("2023-01-02", "Berchtoldstag"),
            ("2023-01-06", "Heilige Drei Könige"),
            ("2023-03-01", "Jahrestag der Ausrufung der Republik"),
            ("2023-03-19", "Josefstag"),
            ("2023-04-06", "Vortag vor Karfreitag"),
            ("2023-04-07", "Karfreitag"),
            ("2023-04-09", "Ostersonntag"),
            ("2023-04-10", "Ostermontag"),
            ("2023-04-13", "Näfelser Fahrt"),
            ("2023-04-17", "Sechseläuten"),
            ("2023-05-01", "Tag der Arbeit"),
            ("2023-05-17", "Vortag vor Auffahrt"),
            ("2023-05-18", "Auffahrt"),
            ("2023-05-28", "Pfingstsonntag"),
            ("2023-05-29", "Pfingstmontag"),
            ("2023-06-08", "Fronleichnam"),
            ("2023-06-23", "Fest der Unabhängigkeit"),
            ("2023-06-29", "Peter und Paul"),
            ("2023-08-01", "Nationalfeiertag"),
            ("2023-08-15", "Mariä Himmelfahrt"),
            ("2023-09-07", "Genfer Bettag"),
            ("2023-09-09", "Knabenschiessen"),
            ("2023-09-10", "Knabenschiessen"),
            ("2023-09-11", "Knabenschiessen"),
            ("2023-09-17", "Eidgenössischer Bettag"),
            ("2023-09-18", "Bettagsmontag"),
            ("2023-09-25", "Bruder Klaus"),
            ("2023-11-01", "Allerheiligen"),
            ("2023-12-08", "Mariä Empfängnis"),
            ("2023-12-24", "Heiligabend"),
            ("2023-12-25", "Weihnachten"),
            ("2023-12-26", "Stephanstag"),
            ("2023-12-27", "Brückentag"),
            ("2023-12-28", "Brückentag"),
            ("2023-12-29", "Brückentag"),
            ("2023-12-31", "Vortag vor Neujahr; Wiederherstellung der Republik"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "Saint Berchtold's Day"),
            ("2023-01-06", "Epiphany"),
            ("2023-03-01", "Republic Day"),
            ("2023-03-19", "Saint Joseph's Day"),
            ("2023-04-06", "Day before Good Friday"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-09", "Easter Sunday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-04-13", "Battle of Naefels Victory Day"),
            ("2023-04-17", "Sechseläuten"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-17", "Day before Ascension Day"),
            ("2023-05-18", "Ascension Day"),
            ("2023-05-28", "Whit Sunday"),
            ("2023-05-29", "Whit Monday"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-23", "Independence Day"),
            ("2023-06-29", "Saints Peter and Paul"),
            ("2023-08-01", "National Day"),
            ("2023-08-15", "Assumption Day"),
            ("2023-09-07", "Genevan Fast"),
            ("2023-09-09", "Knabenschiessen"),
            ("2023-09-10", "Knabenschiessen"),
            ("2023-09-11", "Knabenschiessen"),
            ("2023-09-17", "Federal Day of Thanksgiving, Repentance, and Prayer"),
            ("2023-09-18", "Prayer Monday"),
            ("2023-09-25", "Saint Nicholas of Flüe"),
            ("2023-11-01", "All Saints' Day"),
            ("2023-12-08", "Immaculate Conception"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Saint Stephen's Day"),
            ("2023-12-27", "Bridge Holiday"),
            ("2023-12-28", "Bridge Holiday"),
            ("2023-12-29", "Bridge Holiday"),
            ("2023-12-31", "New Year's Eve; Restoration Day"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2023-01-01", "Nouvel An"),
            ("2023-01-02", "Saint-Berchtold"),
            ("2023-01-06", "Épiphanie"),
            ("2023-03-01", "Instauration de la République"),
            ("2023-03-19", "Saint-Joseph"),
            ("2023-04-06", "Veille du Vendredi saint"),
            ("2023-04-07", "Vendredi saint"),
            ("2023-04-09", "Jour de Pâques"),
            ("2023-04-10", "Lundi de Pâques"),
            ("2023-04-13", "Fahrtsfest"),
            ("2023-04-17", "Sechseläuten"),
            ("2023-05-01", "Fête du Travail"),
            ("2023-05-17", "Veille de l'Ascension"),
            ("2023-05-18", "Ascension"),
            ("2023-05-28", "Jour de Pentecôte"),
            ("2023-05-29", "Lundi de Pentecôte"),
            ("2023-06-08", "Fête-Dieu"),
            ("2023-06-23", "Commémoration du plébiscite"),
            ("2023-06-29", "Saint-Pierre et Paul"),
            ("2023-08-01", "Fête nationale"),
            ("2023-08-15", "Assomption"),
            ("2023-09-07", "Jeûne genevois"),
            ("2023-09-09", "Knabenschiessen"),
            ("2023-09-10", "Knabenschiessen"),
            ("2023-09-11", "Knabenschiessen"),
            ("2023-09-17", "Jeûne fédéral"),
            ("2023-09-18", "Lundi du Jeûne fédéral"),
            ("2023-09-25", "Fête de Saint-Nicolas-de-Flüe"),
            ("2023-11-01", "Toussaint"),
            ("2023-12-08", "Immaculée Conception"),
            ("2023-12-24", "Veille de Noël"),
            ("2023-12-25", "Noël"),
            ("2023-12-26", "Saint-Étienne"),
            ("2023-12-27", "Jour pont"),
            ("2023-12-28", "Jour pont"),
            ("2023-12-29", "Jour pont"),
            ("2023-12-31", "Restauration de la République; Réveillon du Nouvel An"),
        )

    def test_l10n_it(self):
        self.assertLocalizedHolidays(
            "it",
            ("2023-01-01", "Capodanno"),
            ("2023-01-02", "Giorno di Bertoldo"),
            ("2023-01-06", "Epifania"),
            ("2023-03-01", "Giorno della Repubblica"),
            ("2023-03-19", "San Giuseppe"),
            ("2023-04-06", "Vigilia del Venerdì Santo"),
            ("2023-04-07", "Venerdì Santo"),
            ("2023-04-09", "Pasqua"),
            ("2023-04-10", "Lunedì dell'Angelo"),
            ("2023-04-13", "Battaglia di Näfels"),
            ("2023-04-17", "Sechseläuten"),
            ("2023-05-01", "Festa del lavoro"),
            ("2023-05-17", "Vigilia dell'Ascensione di Gesù"),
            ("2023-05-18", "Ascensione di Gesù"),
            ("2023-05-28", "Domenica di Pentecoste"),
            ("2023-05-29", "Lunedì di Pentecoste"),
            ("2023-06-08", "Corpus Domini"),
            ("2023-06-23", "Festa dell'Indipendenza"),
            ("2023-06-29", "Santi Pietro e Paolo"),
            ("2023-08-01", "Festa nazionale"),
            ("2023-08-15", "Assunzione di Maria"),
            ("2023-09-07", "Digiuno ginevrino"),
            ("2023-09-09", "Knabenschiessen"),
            ("2023-09-10", "Knabenschiessen"),
            ("2023-09-11", "Knabenschiessen"),
            ("2023-09-17", "Digiuno federale"),
            ("2023-09-18", "Lunedì del digiuno federale"),
            ("2023-09-25", "San Nicolao della Flue"),
            ("2023-11-01", "Ognissanti"),
            ("2023-12-08", "Immacolata Concezione"),
            ("2023-12-24", "Vigilia di Natale"),
            ("2023-12-25", "Natale"),
            ("2023-12-26", "Giorno di Santo Stefano"),
            ("2023-12-27", "Giorno ponte"),
            ("2023-12-28", "Giorno ponte"),
            ("2023-12-29", "Giorno ponte"),
            ("2023-12-31", "Restaurazione della Repubblica; Vigilia di Capodanno"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2023-01-01", "วันขึ้นปีใหม่"),
            ("2023-01-02", "วันสมโภชนักบุญแบร์กโทลด์"),
            ("2023-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2023-03-01", "วันครบรอบการสถาปนาสาธารณรัฐนอยชาแตล"),
            ("2023-03-19", "วันสมโภชนักบุญโยเซฟ"),
            ("2023-04-06", "วันก่อนวันศุกร์ประเสริฐ"),
            ("2023-04-07", "วันศุกร์ประเสริฐ"),
            ("2023-04-09", "วันอาทิตย์อีสเตอร์"),
            ("2023-04-10", "วันจันทร์อีสเตอร์"),
            ("2023-04-13", "วันรำลึกชัยชนะยุทธการเนเฟลส์"),
            ("2023-04-17", "เซ็กเซ่ะเล๊าเท่น"),
            ("2023-05-01", "วันแรงงาน"),
            ("2023-05-17", "วันก่อนวันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2023-05-18", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2023-05-28", "วันสมโภชพระจิตเจ้า"),
            ("2023-05-29", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2023-06-08", "วันสมโภชพระคริสตวรกาย"),
            ("2023-06-23", "วันประกาศเอกราชจูรา"),
            ("2023-06-29", "วันสมโภชนักบุญเปโตรและเปาโล"),
            ("2023-08-01", "วันชาติสวิตเซอร์แลนด์"),
            ("2023-08-15", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2023-09-07", "วันถือศีลอดเจนีวา"),
            ("2023-09-09", "คนาเบนชิสเซน"),
            ("2023-09-10", "คนาเบนชิสเซน"),
            ("2023-09-11", "คนาเบนชิสเซน"),
            ("2023-09-17", "วันอธิษฐานแห่งชาติสวิตเซอร์แลนด์"),
            ("2023-09-18", "วันจันทร์หลังวันอธิษฐานแห่งชาติสวิตเซอร์แลนด์"),
            ("2023-09-25", "วันสมโภชนักบุญนิโคลัสแห่งฟลือเออ"),
            ("2023-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2023-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2023-12-24", "วันคริสต์มาสอีฟ"),
            ("2023-12-25", "วันคริสต์มาส"),
            ("2023-12-26", "วันสมโภชนักบุญสเตเฟน"),
            ("2023-12-27", "วันหยุดเพิ่มเติม"),
            ("2023-12-28", "วันหยุดเพิ่มเติม"),
            ("2023-12-29", "วันหยุดเพิ่มเติม"),
            ("2023-12-31", "วันกอบกู้เอกราชสาธารณรัฐเจนีวา; วันสิ้นปี"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-02", "День Святого Бертольда"),
            ("2023-01-06", "Богоявлення"),
            ("2023-03-01", "Річниця проголошення Республіки"),
            ("2023-03-19", "День Святого Йосипа"),
            ("2023-04-06", "Переддень Страсної пʼятниці"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-04-09", "Великдень"),
            ("2023-04-10", "Великодній понеділок"),
            ("2023-04-13", "Свято перемоги під Нефельсом"),
            ("2023-04-17", "Зексельйотен"),
            ("2023-05-01", "День праці"),
            ("2023-05-17", "Переддень Вознесіння Господнього"),
            ("2023-05-18", "Вознесіння Господнє"),
            ("2023-05-28", "Трійця"),
            ("2023-05-29", "День Святого Духа"),
            ("2023-06-08", "Свято Тіла і Крові Христових"),
            ("2023-06-23", "День незалежності"),
            ("2023-06-29", "День Святих Петра і Павла"),
            ("2023-08-01", "Національне свято"),
            ("2023-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2023-09-07", "Женевський піст"),
            ("2023-09-09", "Кнабеншісен"),
            ("2023-09-10", "Кнабеншісен"),
            ("2023-09-11", "Кнабеншісен"),
            ("2023-09-17", "Швейцарський національний день молитви"),
            ("2023-09-18", "Молитовний понеділок"),
            ("2023-09-25", "День Святого Ніклауса з Флюе"),
            ("2023-11-01", "День усіх святих"),
            ("2023-12-08", "Непорочне зачаття Діви Марії"),
            ("2023-12-24", "Святий вечір"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-26", "День Святого Стефана"),
            ("2023-12-27", "Проміжний вихідний"),
            ("2023-12-28", "Проміжний вихідний"),
            ("2023-12-29", "Проміжний вихідний"),
            ("2023-12-31", "День відновлення республіки; Переддень Нового року"),
        )



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from concurrent.futures import ProcessPoolExecutor
from pathlib import Path
from time import perf_counter

from polib import pofile


class MOGenerator:
    """Creates .mo files for supported country/market entities."""

    @staticmethod
    def _convert_po_to_mo(po_path: Path) -> None:
        """Convert a single .po file to .mo file."""
        pofile(po_path).save_as_mofile(str(po_path.with_suffix(".mo")))

    @staticmethod
    def _unlink_file(path: Path) -> None:
        """Unlink old .mo file"""
        path.unlink(missing_ok=True)

    @staticmethod
    def run() -> None:
        """Runs the .mo files generation process."""
        locale_path = Path("holidays/locale")
        with ProcessPoolExecutor() as executor:
            list(executor.map(MOGenerator._unlink_file, locale_path.rglob("*.mo")))
            list(executor.map(MOGenerator._convert_po_to_mo, locale_path.rglob("*.po")))


if __name__ == "__main__":
    mo_time_start = perf_counter()
    MOGenerator.run()
    mo_time_end = perf_counter()
    print(f"[TIMER] Total generate mo files runtime: {mo_time_end - mo_time_start:.2f} seconds")



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate Gregorian dates for holidays based on the Mongolian lunisolar calendar.

References:
    * <https://en.wikipedia.org/wiki/Mongolian_calendar>
    * <https://web.archive.org/web/20250606051130/https://www.math.mcgill.ca/gantumur/cal/year.html>

Run with:

    python -m scripts.calendar.mongolian_generator

Alternatively, run with uv:

    uv run -m scripts.calendar.mongolian_generator

This generates the file `holidays/calendars/mongolian_dates.py`,
whose data can then be copied to `holidays/calendars/mongolian.py`.
"""

from collections import defaultdict
from datetime import date

from .generator import CalendarGenerator

m_zero = 3
epoch = 1747
ixx = 46
betastar = 10
beta = 172

m0 = 2359237 + 2603 / 2828
m1 = 167025 / 5656
m2 = 11135 / 11312
s0 = 397 / 402
s1 = 65 / 804
s2 = 13 / 4824
a0 = 1523 / 1764
a1 = 253 / 3528
a2 = 1 / 28


def mstar(y: int, m: int) -> int:
    return 12 * (y - epoch) + m - m_zero


def leap_month(y: int, m: int) -> bool:
    t = (24 * (y - epoch) + 2 * m - beta) % 65
    return t <= 1


def leap_year(y: int) -> bool:
    t = (24 * (y - epoch) - beta) % 65
    return t >= 41


def true_month(y: int, m: int, *, is_leap: bool) -> int:
    p = 67 * mstar(y, m) + betastar
    pp, ix = divmod(p, 65)
    return pp if is_leap or ix < ixx else pp + 1


def moon_tab(i: float) -> float:
    i %= 28
    s = 1
    if i >= 14:
        i -= 14
        s = -1
    if i > 7:
        i = 14 - i
    a = int(i)
    b = a + 1
    v = (0, 5, 10, 15, 19, 22, 24, 25)
    return s * (v[a] if a == i else (b - i) * v[a] + (i - a) * v[b])


def sun_tab(i: float) -> float:
    i %= 12
    s = 1
    if i >= 6:
        i -= 6
        s = -1
    if i > 3:
        i = 6 - i
    a = int(i)
    b = a + 1
    v = (0, 6, 10, 11)
    return s * (v[a] if a == i else (b - i) * v[a] + (i - a) * v[b])


def true_date(d: int, n: int) -> float:
    mean_date = n * m1 + d * m2 + m0
    mean_sun = n * s1 + d * s2 + s0
    anomaly_moon = n * a1 + d * a2 + a0
    moon_equ = moon_tab(28 * anomaly_moon)
    anomaly_sun = mean_sun - 0.25
    sun_equ = sun_tab(12 * anomaly_sun)
    return mean_date + moon_equ / 60 - sun_equ / 60


def julian_day(y: int, m: int, d: int, *, is_leap: bool = False) -> int:
    n = true_month(y, m, is_leap=is_leap)
    t = true_date(d, n)
    return int(t)


def gregorian_date(jdn: int) -> date:
    f = jdn + 1401 + (4 * jdn + 274277) // 146097 * 3 // 4 - 38
    e = 4 * f + 3
    h = e % 1461 // 4
    h = 5 * h + 2
    d = (h % 153) // 5 + 1
    m = (h // 153 + 2) % 12 + 1
    y = e // 1461 - 4716 + (14 - m) // 12
    return date(y, m, d)


def mongolian_to_gregorian(m_year: int, m_month: int, m_day: int, *, is_leap: bool) -> date:
    return gregorian_date(julian_day(m_year, m_month, m_day, is_leap=is_leap))


def get_tsagaan_sar(y: int) -> date:
    return gregorian_date(julian_day(y - 1, 12, 30) + 1)


def find_festival_date(y: int, m_month: int, m_day: int) -> date:
    return gregorian_date(julian_day(y, m_month, m_day))


MONGOLIAN_HOLIDAYS = {
    "BUDDHA_DAY": lambda y: find_festival_date(y, 4, 15),
    "GENGHIS_KHAN_DAY": lambda y: find_festival_date(y, 10, 1),
    "TSAGAAN_SAR": get_tsagaan_sar,
}


def generate_data() -> None:
    years = range(2004, 2101)

    dates: dict[str, dict[int, date]] = defaultdict(dict)
    for name, holiday_func in MONGOLIAN_HOLIDAYS.items():
        for year in years:
            dates[name][year] = holiday_func(year)

    cal_gen = CalendarGenerator("mongolian", "_MongolianLunisolar")
    cal_gen.generate(dates)


if __name__ == "__main__":
    generate_data()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)


class PreferredDiscretionaryHolidays:
    """A mixin for setting preferred discretionary holidays.

    See [HongKong][holidays.countries.hongkong.HongKong] for an example.
    """

    default_preferred_discretionary_holidays: tuple[str, ...] = ()
    """Preferred discretionary holidays defaults."""

    def __init__(self, preferred_discretionary_holidays):
        self.preferred_discretionary_holidays = set(
            preferred_discretionary_holidays
            if preferred_discretionary_holidays
            else self.default_preferred_discretionary_holidays
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.mexico import Mexico
from tests.common import CommonCountryTests


class TestMexico(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Mexico, years=range(1901, 2050))

    def test_new_years_day(self):
        self.assertHoliday(f"{year}-01-01" for year in range(1901, 2050))

    def test_constitution_day(self):
        self.assertHoliday(f"{year}-02-05" for year in range(1917, 2006))
        self.assertNoHoliday(f"{year}-02-05" for year in range(1901, 1917))
        self.assertNoHolidayName("Día de la Constitución", range(1901, 1917))
        self.assertHoliday(
            "2006-02-06",
            "2007-02-05",
            "2008-02-04",
            "2009-02-02",
            "2010-02-01",
            "2011-02-07",
            "2012-02-06",
            "2013-02-04",
            "2014-02-03",
            "2015-02-02",
            "2016-02-01",
            "2017-02-06",
            "2018-02-05",
            "2019-02-04",
            "2020-02-03",
            "2021-02-01",
            "2022-02-07",
            "2023-02-06",
        )

    def test_benito_juarez(self):
        self.assertHoliday(f"{year}-03-21" for year in range(1917, 2007))
        self.assertNoHoliday(f"{year}-03-21" for year in range(1901, 1917))
        self.assertNoHolidayName("Natalicio de Benito Juárez", range(1901, 1917))
        self.assertHoliday(
            "2007-03-19",
            "2008-03-17",
            "2009-03-16",
            "2010-03-15",
            "2011-03-21",
            "2012-03-19",
            "2013-03-18",
            "2014-03-17",
            "2015-03-16",
            "2016-03-21",
            "2017-03-20",
            "2018-03-19",
            "2019-03-18",
            "2020-03-16",
            "2021-03-15",
            "2022-03-21",
            "2023-03-20",
        )

    def test_labour_day(self):
        self.assertHoliday(f"{year}-05-01" for year in range(1923, 2050))
        self.assertNoHoliday(f"{year}-05-01" for year in range(1901, 1923))
        self.assertNoHolidayName("Día del Trabajo", range(1901, 1923))

    def test_independence_day(self):
        self.assertHoliday(f"{year}-09-16" for year in range(1901, 2050))

    def test_revolution_day(self):
        self.assertHoliday(f"{year}-11-20" for year in range(1917, 2006))
        self.assertNoHoliday(f"{year}-11-20" for year in range(1901, 1917))
        self.assertNoHolidayName("Día de la Revolución", range(1901, 1917))
        self.assertHoliday(
            "2006-11-20",
            "2007-11-19",
            "2008-11-17",
            "2009-11-16",
            "2010-11-15",
            "2011-11-21",
            "2012-11-19",
            "2013-11-18",
            "2014-11-17",
            "2015-11-16",
            "2016-11-21",
            "2017-11-20",
            "2018-11-19",
            "2019-11-18",
            "2020-11-16",
            "2021-11-15",
            "2022-11-21",
            "2023-11-20",
        )

    def test_change_of_government(self):
        self.assertHoliday(
            "1970-12-01",
            "1976-12-01",
            "1982-12-01",
            "1988-12-01",
            "1994-12-01",
            "2000-12-01",
            "2006-12-01",
            "2012-12-01",
            "2018-12-01",
            "2024-10-01",
        )
        self.assertNoHoliday(
            f"{year}-12-01" for year in range(1970, 2050) if (year - 1970) % 6 > 0
        )
        name = "Transmisión del Poder Ejecutivo Federal"
        self.assertNoHolidayName(name, range(1901, 1970))
        self.assertNoHolidayName(
            name, (year for year in range(1970, 2050) if (year - 1970) % 6 > 0)
        )
        self.assertNoHoliday("2024-12-01")

    def test_christmas_day(self):
        self.assertHoliday(f"{year}-12-25" for year in range(1901, 2050))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-02-07", "Día de la Constitución"),
            ("2022-03-21", "Natalicio de Benito Juárez"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-09-16", "Día de la Independencia"),
            ("2022-11-21", "Día de la Revolución"),
            ("2022-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-07", "Constitution Day"),
            ("2022-03-21", "Benito Juárez's birthday"),
            ("2022-05-01", "Labor Day"),
            ("2022-09-16", "Independence Day"),
            ("2022-11-21", "Revolution Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-07", "День Конституції"),
            ("2022-03-21", "Річниця Беніто Хуареса"),
            ("2022-05-01", "День праці"),
            ("2022-09-16", "День незалежності"),
            ("2022-11-21", "День революції"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.british_virgin_islands import BritishVirginIslands
from tests.common import CommonCountryTests


class TestBritishVirginIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(BritishVirginIslands)

    def test_special_holidays(self):
        self.assertHoliday(
            "2019-12-11",
            "2022-06-03",
            "2023-05-08",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_lavity_stoutts_birthday(self):
        name = "The Anniversary of the Birth of Hamilton Lavity Stoutt"
        self.assertHolidayName(
            name,
            "2020-03-02",
            "2021-03-01",
            "2022-03-07",
            "2023-03-06",
            "2024-03-04",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_commonwealth_day(self):
        name = "Commonwealth Day"
        self.assertHolidayName(
            name,
            "2015-03-09",
            "2016-03-14",
            "2017-03-13",
            "2018-03-12",
            "2019-03-11",
            "2020-03-09",
        )
        self.assertHolidayName(name, range(self.start_year, 2021))
        self.assertNoHolidayName(name, range(2021, self.end_year))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_sovereigns_birthday(self):
        name = "Sovereign's Birthday"
        self.assertHolidayName(
            name,
            "2018-06-09",
            "2019-06-07",
            "2020-06-12",
            "2021-06-11",
            "2022-06-10",
            "2023-06-16",
            "2024-06-14",
            "2025-06-13",
        )
        self.assertHolidayName(name, self.full_range)

    def test_territory_day(self):
        name_1967 = "Colony Day"
        name_1978 = "Territory Day"
        name_2021 = "Virgin Islands Day"
        self.assertHolidayName(
            name_1967, (f"{year}-07-01" for year in range(self.start_year, 1978))
        )
        self.assertHolidayName(
            name_1978,
            (f"{year}-07-01" for year in (*range(1978, 2015), *range(2016, 2020))),
            "2015-06-29",
            "2020-06-29",
        )
        self.assertHolidayName(
            name_2021,
            "2021-07-05",
            "2022-07-04",
            "2023-07-03",
            "2024-07-01",
            "2025-07-07",
        )
        self.assertNoHolidayName(name_1967, range(1978, self.end_year))
        self.assertNoHolidayName(
            name_1978, range(self.start_year, 1978), range(2021, self.end_year)
        )
        self.assertNoHolidayName(name_2021, range(self.start_year, 2021))
        obs_dts = (
            "2007-07-02",
            "2012-07-02",
            "2017-06-30",
            "2018-07-02",
        )
        self.assertHolidayName(f"{name_1978} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_emancipation_monday(self):
        name_1967 = "Festival Monday"
        name_2021 = "Emancipation Monday"
        self.assertHolidayName(
            name_1967,
            "2018-08-06",
            "2019-08-05",
            "2020-08-03",
        )
        self.assertHolidayName(
            name_2021,
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
        )
        self.assertHolidayName(name_1967, range(self.start_year, 2021))
        self.assertHolidayName(name_2021, range(2021, self.end_year))
        self.assertNoHolidayName(name_1967, range(2021, self.end_year))
        self.assertNoHolidayName(name_2021, range(self.start_year, 2021))

    def test_emancipation_tuesday(self):
        name_1967 = "Festival Tuesday"
        name_2021 = "Emancipation Tuesday"
        self.assertHolidayName(
            name_1967,
            "2018-08-07",
            "2019-08-06",
            "2020-08-04",
        )
        self.assertHolidayName(
            name_2021,
            "2021-08-03",
            "2022-08-02",
            "2023-08-08",
            "2024-08-06",
        )
        self.assertHolidayName(name_1967, range(self.start_year, 2021))
        self.assertHolidayName(name_2021, range(2021, self.end_year))
        self.assertNoHolidayName(name_1967, range(2021, self.end_year))
        self.assertNoHolidayName(name_2021, range(self.start_year, 2021))

    def test_emancipation_wednesday(self):
        name_1967 = "Festival Wednesday"
        name_2021 = "Emancipation Wednesday"
        self.assertHolidayName(
            name_1967,
            "2018-08-08",
            "2019-08-07",
            "2020-08-05",
        )
        self.assertHolidayName(
            name_2021,
            "2021-08-04",
            "2022-08-03",
            "2023-08-09",
            "2024-08-07",
        )
        self.assertHolidayName(name_1967, range(self.start_year, 2021))
        self.assertHolidayName(name_2021, range(2021, self.end_year))
        self.assertNoHolidayName(name_1967, range(2021, self.end_year))
        self.assertNoHolidayName(name_2021, range(self.start_year, 2021))

    def test_saint_ursulas_day(self):
        name = "Saint Ursula's Day"
        self.assertHolidayName(
            name,
            (f"{year}-10-21" for year in (*range(self.start_year, 2015), *range(2016, 2020))),
            "2015-10-19",
            "2020-10-23",
        )
        self.assertNoHolidayName(name, range(2021, self.end_year))
        obs_dts = (
            "2007-10-22",
            "2012-10-22",
            "2017-10-20",
            "2018-10-22",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_heroes_and_foreparents_day(self):
        name = "Heroes and Foreparents Day"
        self.assertHolidayName(
            name,
            "2021-10-18",
            "2022-10-17",
            "2023-10-16",
            "2024-10-21",
            "2025-10-20",
        )
        self.assertHolidayName(name, range(2021, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2021))

    def test_great_march_and_restoration_day(self):
        name = "The Great March of 1949 and Restoration Day"
        self.assertHolidayName(
            name,
            "2021-11-22",
            "2022-11-28",
            "2023-11-27",
            "2024-11-25",
            "2025-11-24",
        )
        self.assertHolidayName(name, range(2021, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2021))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-03-07", "The Anniversary of the Birth of Hamilton Lavity Stoutt"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-06-03", "Queen Elizabeth II's Platinum Jubilee"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-10", "Sovereign's Birthday"),
            ("2022-07-04", "Virgin Islands Day"),
            ("2022-08-01", "Emancipation Monday"),
            ("2022-08-02", "Emancipation Tuesday"),
            ("2022-08-03", "Emancipation Wednesday"),
            ("2022-10-17", "Heroes and Foreparents Day"),
            ("2022-11-28", "The Great March of 1949 and Restoration Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-03-07", "Lavity Stoutt's Birthday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-06-03", "Platinum Jubilee of Elizabeth II"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-10", "Sovereign's Birthday"),
            ("2022-07-04", "Virgin Islands Day"),
            ("2022-08-01", "Emancipation Monday"),
            ("2022-08-02", "Emancipation Tuesday"),
            ("2022-08-03", "Emancipation Wednesday"),
            ("2022-10-17", "Heroes and Foreparents Day"),
            ("2022-11-28", "The Great March of 1949 and Restoration Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.constants import BANK, PUBLIC
from holidays.countries.south_korea import SouthKorea, Korea
from tests.common import CommonCountryTests


class TestSouthKorea(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SouthKorea)

    def test_special_holidays(self):
        # Election Dates have its own separate checklists.
        self.assertHoliday(
            "1948-12-15",
            "1949-05-10",
            "1949-07-05",
            "1950-06-21",
            "1957-03-26",
            "1958-03-26",
            "1959-03-26",
            "1960-03-16",
            "1960-03-26",
            "1960-10-01",
            "1961-04-19",
            "1962-04-19",
            "1962-05-16",
            "1963-04-19",
            "1963-05-16",
            "1963-12-17",
            "1966-10-01",
            "1967-01-04",
            "1967-07-01",
            "1969-07-21",
            "1969-10-17",
            "1971-07-01",
            "1972-11-21",
            "1972-12-15",
            "1972-12-27",
            "1974-08-19",
            "1975-02-12",
            "1978-05-18",
            "1978-12-27",
            "1979-11-03",
            "1979-12-21",
            "1980-09-01",
            "1980-10-22",
            "1981-02-11",
            "1981-03-03",
            "1982-10-02",
            "1987-10-27",
            "1988-02-25",
            "1988-09-17",
            "2002-07-01",
            "2017-05-09",
            "2017-10-02",
            "2020-08-17",
            "2023-10-02",
            "2024-10-01",
            "2025-01-27",
        )
        # Pre-2014 Observance sans "1960-12-26"
        self.assertNoNonObservedHoliday(
            "1959-04-06",
            "1960-07-18",
            "1960-10-10",
            "1989-10-02",
        )

    def test_national_assembly_election_day(self):
        self.assertHoliday(
            # 1st Republic.
            "1948-05-10",
            "1950-05-30",
            "1954-05-20",
            "1958-05-02",
            "1960-07-29",
            # No Election during the short 2nd Republic.
            # 3rd Republic.
            "1963-11-26",
            "1967-06-08",
            "1971-05-25",
            # 4th Republic.
            "1973-02-27",
            "1978-12-12",
            # 5th Republic.
            "1981-03-25",
            "1985-02-12",
            # 6th Republic.
            "1988-04-26",
            "1992-03-24",
            "1996-04-11",
            "2000-04-13",
            "2004-04-15",
            # Codified rather than on ad-hoc basis from SEP 2006 onwards.
            "2008-04-09",
            "2012-04-11",
            "2016-04-13",
            "2020-04-15",
            # Preliminary Dates.
            "2024-04-10",
            "2028-04-12",
        )

    def test_presidential_election_day(self):
        self.assertHoliday(
            # 1st Republic. (incl. 2nd Vice President Election in 1951)
            "1948-07-20",
            "1951-05-16",
            "1952-08-05",
            "1956-05-15",
            "1960-03-15",
            # 2nd Republic.
            "1960-08-12",
            # 3rd Republic.
            "1963-10-15",
            "1967-05-03",
            "1971-04-27",
            # 4th Republic.
            "1972-12-23",
            "1978-07-06",
            "1979-12-06",
            "1980-08-27",
            # 5th Republic.
            "1981-02-25",
            # 6th Republic.
            "1987-12-16",
            "1992-12-18",
            "1997-12-18",
            "2002-12-19",
            # Codified rather than on ad-hoc basis from SEP 2006 onwards.
            "2007-12-19",
            "2012-12-19",
            "2017-05-09",
            "2022-03-09",
            # Special Presidential Election (21st) due to Yoon Seok-yeol's impeachment.
            "2025-06-03",
            "2030-04-03",
        )

    def test_local_election_day(self):
        self.assertHoliday(
            # 1st Republic Local Elections.
            "1952-04-25",
            "1952-05-10",
            "1956-08-08",
            "1956-08-13",
            "1960-12-12",
            "1960-12-19",
            "1960-12-26",
            "1960-12-29",
            # 1991 Local Elections.
            "1991-03-26",
            "1991-06-20",
            # Nationwide Local Elections.
            "1995-06-27",
            "1998-06-04",
            "2002-06-13",
            "2006-05-31",
            # Codified rather than on ad-hoc basis from SEP 2006 onwards.
            "2010-06-02",
            "2014-06-04",
            "2018-06-13",
            "2022-06-01",
            # Preliminary Dates.
            "2026-06-03",
            "2030-06-12",
        )

    def test_common(self):
        self.assertNonObservedHolidayName("신정연휴", "2019-01-01")

    def test_new_years_day(self):
        name = "신정연휴"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in range(self.start_year, 1999)),
            (f"{year}-01-03" for year in range(self.start_year, 1990)),
        )
        self.assertNoHolidayName(
            name,
            (f"{year}-01-02" for year in range(1999, self.end_year)),
            (f"{year}-01-03" for year in range(1990, self.end_year)),
        )

    def test_korean_new_years_day(self):
        name_1985 = "민속의 날"
        name_1989 = "설날"
        name_1989_eve = f"{name_1989} 전날"
        name_1989_day2 = f"{name_1989} 다음날"
        name_1989_observed = f"{name_1989} 대체 휴일"
        self.assertHolidayName(
            name_1985,
            "1985-02-20",
            "1986-02-09",
            "1987-01-29",
            "1988-02-18",
        )
        self.assertHolidayName(
            name_1989,
            "2020-01-25",
            "2021-02-12",
            "2022-02-01",
            "2023-01-22",
            "2024-02-10",
            "2025-01-29",
        )
        self.assertHolidayName(name_1989, range(1989, self.end_year))
        self.assertHolidayName(
            name_1989_eve,
            "2020-01-24",
            "2021-02-11",
            "2022-01-31",
            "2023-01-21",
            "2024-02-09",
            "2025-01-28",
        )
        self.assertHolidayName(name_1989_eve, range(1989, self.end_year))
        self.assertHolidayName(
            name_1989_day2,
            "2020-01-26",
            "2021-02-13",
            "2022-02-02",
            "2023-01-23",
            "2024-02-11",
            "2025-01-30",
        )
        self.assertHolidayName(name_1989_day2, range(1989, self.end_year))
        self.assertHolidayName(
            name_1989_observed,
            "2016-02-10",
            "2017-01-30",
            "2020-01-27",
            "2023-01-24",
            "2024-02-12",
            "2027-02-09",
        )
        self.assertNoHolidayName(
            name_1985, range(self.start_year, 1985), range(1989, self.end_year)
        )
        self.assertNoHolidayName(f"{name_1985} 전날")
        self.assertNoHolidayName(f"{name_1985} 다음날")
        self.assertNoHolidayName(f"{name_1985} 대체 휴일")
        self.assertNoHolidayName(name_1989, range(self.start_year, 1989))
        self.assertNoHolidayName(name_1989_eve, range(self.start_year, 1989))
        self.assertNoHolidayName(name_1989_day2, range(self.start_year, 1989))
        self.assertNoHolidayName(name_1989_observed, range(self.start_year, 2014))

    def test_independence_movement_day(self):
        name = "삼일절"
        self.assertHolidayName(name, (f"{year}-03-01" for year in self.full_range))
        self.assertHolidayName(
            f"{name} 대체 휴일",
            "2025-03-03",
            "2026-03-02",
        )

    def test_tree_planting_day(self):
        name = "식목일"
        self.assertHolidayName(
            name, (f"{year}-04-05" for year in (*range(1949, 1960), *range(1961, 2006)))
        )
        self.assertNoHolidayName(
            name, range(self.start_year, 1949), 1960, range(2006, self.end_year)
        )
        self.assertHolidayName(f"{name} 대체 휴일", "1959-04-06")

    def test_childrens_day(self):
        name = "어린이날"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(name, (f"{year}-05-05" for year in range(1975, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1975))
        self.assertHolidayName(
            name_observed,
            "2018-05-07",
            "2019-05-06",
            "2024-05-06",
            "2025-05-06",
            "2029-05-07",
            "2030-05-06",
        )
        self.assertNoHolidayName(name_observed, range(1975, 2014))

    def test_buddhas_birthday(self):
        name_1975 = "석가탄신일"
        name_2017 = "부처님오신날"
        name_2017_observed = f"{name_2017} 대체 휴일"
        self.assertHolidayName(
            name_1975,
            "2006-05-05",
            "2007-05-24",
            "2008-05-12",
            "2009-05-02",
            "2010-05-21",
            "2011-05-10",
            "2012-05-28",
            "2013-05-17",
            "2014-05-06",
            "2015-05-25",
            "2016-05-14",
        )
        self.assertHolidayName(name_1975, range(1975, 2017))
        self.assertHolidayName(
            name_2017,
            "2017-05-03",
            "2018-05-22",
            "2019-05-12",
            "2020-04-30",
            "2021-05-19",
            "2022-05-08",
            "2023-05-27",
            "2024-05-15",
            "2025-05-05",
            "2026-05-24",
            "2027-05-13",
            "2028-05-02",
            "2029-05-20",
        )
        self.assertHolidayName(name_2017, range(2017, self.end_year))
        self.assertHolidayName(
            name_2017_observed,
            "2023-05-29",
            "2026-05-25",
            "2029-05-21",
        )
        self.assertNoHolidayName(
            name_1975, range(self.start_year, 1975), range(2017, self.end_year)
        )
        self.assertNoHolidayName(f"{name_1975} 대체 휴일")
        self.assertNoHolidayName(name_2017, range(self.start_year, 2017))
        self.assertNoHolidayName(name_2017_observed, range(self.start_year, 2023))

    def test_memorial_day(self):
        name = "현충일"
        self.assertHolidayName(name, (f"{year}-06-06" for year in range(1956, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1956))

    def test_constitution_day(self):
        name = "제헌절"
        self.assertHolidayName(
            name,
            (
                f"{year}-07-17"
                for year in (*range(self.start_year, 2008), *range(2026, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(2008, 2026))
        self.assertHolidayName(f"{name} 대체 휴일", "1960-07-18")

    def test_liberation_day(self):
        name = "광복절"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        self.assertHolidayName(
            name_observed,
            "2021-08-16",
            "2026-08-17",
            "2027-08-16",
        )
        self.assertNoHolidayName(name_observed, range(self.start_year, 2021))

    def test_chuseok(self):
        name = "추석"
        name_eve = f"{name} 전날"
        name_day2 = f"{name} 다음날"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(
            name,
            "2020-10-01",
            "2021-09-21",
            "2022-09-10",
            "2023-09-29",
            "2024-09-17",
            "2025-10-06",
        )
        self.assertHolidayName(name, self.full_range)
        self.assertHolidayName(
            name_eve,
            "2020-09-30",
            "2021-09-20",
            "2022-09-09",
            "2023-09-28",
            "2024-09-16",
            "2025-10-05",
        )
        self.assertHolidayName(name_eve, range(1989, self.end_year))
        self.assertHolidayName(
            name_day2,
            "1986-09-19",
            "1987-10-08",
            "1988-09-26",
            "2020-10-02",
            "2021-09-22",
            "2022-09-11",
            "2023-09-30",
            "2024-09-18",
            "2025-10-07",
        )
        self.assertHolidayName(name_day2, range(1986, self.end_year))
        self.assertHolidayName(
            name_observed,
            "2014-09-10",
            "2015-09-29",
            "2018-09-26",
            "2022-09-12",
            "2025-10-08",
            "2029-09-24",
        )
        self.assertNoHolidayName(name_eve, range(self.start_year, 1989))
        self.assertNoHolidayName(name_day2, range(self.start_year, 1986))
        self.assertNoHolidayName(name_observed, range(self.start_year, 2014))

    def test_armed_forces_day(self):
        name = "국군의 날"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(1976, 1991)))
        self.assertHolidayName(name_observed, "1989-10-02")
        self.assertNoHolidayName(name_observed, range(1976, 1989), range(1990, 1992))

    def test_national_foundation_day(self):
        name = "개천절"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(name, (f"{year}-10-03" for year in self.full_range))
        self.assertHolidayName(
            name_observed,
            "2021-10-04",
            "2026-10-05",
            "2027-10-04",
        )
        self.assertNoHolidayName(name_observed, range(self.start_year, 2021))

    def test_hangul_day(self):
        name = "한글날"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(
            name,
            (
                f"{year}-10-09"
                for year in (*range(self.start_year, 1991), *range(2013, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1991, 2013))
        self.assertHolidayName(
            name_observed,
            "1960-10-10",
            "2021-10-11",
            "2022-10-10",
            "2027-10-11",
        )
        self.assertNoHolidayName(name_observed, range(self.start_year, 1960), range(1961, 2021))

    def test_united_nations_day(self):
        name = "국제연합일"
        self.assertHolidayName(name, (f"{year}-10-24" for year in range(1950, 1976)))
        self.assertNoHolidayName(name, range(self.start_year, 1950), range(1976, self.end_year))

    def test_christmas_day(self):
        name = "기독탄신일"
        name_observed = f"{name} 대체 휴일"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        self.assertHolidayName(
            name_observed,
            "1960-12-26",
            "2027-12-27",
            "2032-12-27",
            "2033-12-26",
        )
        self.assertNoHolidayName(name_observed, range(self.start_year, 1960), range(1961, 2023))

    def test_workers_day(self):
        self.assertBankHolidayName(
            "근로자의날",
            (f"{year}-03-10" for year in range(self.start_year, 1994)),
            (f"{year}-05-01" for year in range(1994, self.end_year)),
        )

    def test_korea_deprecation_warning(self):
        warnings.simplefilter("default")
        with self.assertWarns(DeprecationWarning):
            Korea()

    def test_2020_all(self):
        self.assertHolidays(
            SouthKorea(categories=(BANK, PUBLIC), years=2020),
            ("2020-01-01", "신정연휴"),
            ("2020-01-24", "설날 전날"),
            ("2020-01-25", "설날"),
            ("2020-01-26", "설날 다음날"),
            ("2020-01-27", "설날 대체 휴일"),
            ("2020-03-01", "삼일절"),
            ("2020-04-15", "국회의원 선거일"),
            ("2020-04-30", "부처님오신날"),
            ("2020-05-01", "근로자의날"),
            ("2020-05-05", "어린이날"),
            ("2020-06-06", "현충일"),
            ("2020-08-15", "광복절"),
            ("2020-08-17", "임시공휴일"),
            ("2020-09-30", "추석 전날"),
            ("2020-10-01", "추석"),
            ("2020-10-02", "추석 다음날"),
            ("2020-10-03", "개천절"),
            ("2020-10-09", "한글날"),
            ("2020-12-25", "기독탄신일"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "신정연휴"),
            ("2022-01-31", "설날 전날"),
            ("2022-02-01", "설날"),
            ("2022-02-02", "설날 다음날"),
            ("2022-03-01", "삼일절"),
            ("2022-03-09", "대통령 선거일"),
            ("2022-05-01", "근로자의날"),
            ("2022-05-05", "어린이날"),
            ("2022-05-08", "부처님오신날"),
            ("2022-06-01", "지방선거일"),
            ("2022-06-06", "현충일"),
            ("2022-08-15", "광복절"),
            ("2022-09-09", "추석 전날"),
            ("2022-09-10", "추석"),
            ("2022-09-11", "추석 다음날"),
            ("2022-09-12", "추석 대체 휴일"),
            ("2022-10-03", "개천절"),
            ("2022-10-09", "한글날"),
            ("2022-10-10", "한글날 대체 휴일"),
            ("2022-12-25", "기독탄신일"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-31", "The day preceding Korean New Year"),
            ("2022-02-01", "Korean New Year"),
            ("2022-02-02", "The second day of Korean New Year"),
            ("2022-03-01", "Independence Movement Day"),
            ("2022-03-09", "Presidential Election Day"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-05", "Children's Day"),
            ("2022-05-08", "Buddha's Birthday"),
            ("2022-06-01", "Local Election Day"),
            ("2022-06-06", "Memorial Day"),
            ("2022-08-15", "Liberation Day"),
            ("2022-09-09", "The day preceding Chuseok"),
            ("2022-09-10", "Chuseok"),
            ("2022-09-11", "The second day of Chuseok"),
            ("2022-09-12", "Alternative holiday for Chuseok"),
            ("2022-10-03", "National Foundation Day"),
            ("2022-10-09", "Hangul Day"),
            ("2022-10-10", "Alternative holiday for Hangul Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันปีใหม่สากล"),
            ("2022-01-31", "วันก่อนเทศกาลซอลลัล"),
            ("2022-02-01", "เทศกาลซอลลัล"),
            ("2022-02-02", "วันหลังเทศกาลซอลลัล"),
            ("2022-03-01", "วันอิสรภาพ"),
            ("2022-03-09", "วันเลือกตั้งประธานาธิบดี"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-05", "วันเด็ก"),
            ("2022-05-08", "วันวิสาขบูชา"),
            ("2022-06-01", "วันเลือกตั้งท้องถิ่น"),
            ("2022-06-06", "วันรำลึกวีรชน"),
            ("2022-08-15", "วันฉลองอิสรภาพ"),
            ("2022-09-09", "วันก่อนเทศกาลชูซอก"),
            ("2022-09-10", "เทศกาลชูซอก"),
            ("2022-09-11", "วันหลังเทศกาลชูซอก"),
            ("2022-09-12", "ชดเชยเทศกาลชูซอก"),
            ("2022-10-03", "วันสถาปนาประเทศ"),
            ("2022-10-09", "วันฮันกึล"),
            ("2022-10-10", "ชดเชยวันฮันกึล"),
            ("2022-12-25", "วันคริสต์มาส"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

import holidays
from holidays.countries.eswatini import Eswatini
from tests.common import CommonCountryTests


class TestEswatini(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Eswatini)

    def test_special_holidays(self):
        self.assertHolidayName("Y2K changeover", "1999-12-31", "2000-01-03")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

        obs_dts = (
            "2023-01-02",
            "2034-01-02",
            "2040-01-02",
            "2045-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(name, (f"{year}-04-19" for year in range(1987, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1987))

        obs_dts = (
            "2026-04-20",
            "2037-04-20",
            "2043-04-20",
            "2048-04-20",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_workers_day(self):
        name = "Worker's Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))

        obs_dts = (
            "2022-05-02",
            "2033-05-02",
            "2039-05-02",
            "2044-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_flag_day(self):
        name = "National Flag Day"
        self.assertHolidayName(name, (f"{year}-04-25" for year in range(1969, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1969))

        obs_dts = (
            "2021-04-26",
            "2027-04-26",
            "2032-04-26",
            "2038-04-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_ascension_day(self):
        name = "Ascension Day"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_birthday_of_late_king_sobhuza(self):
        name = "Birthday of Late King Sobhuza"
        self.assertHolidayName(name, (f"{year}-07-22" for year in range(1983, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1983))

        obs_dts = (
            "2029-07-23",
            "2035-07-23",
            "2040-07-23",
            "2046-07-23",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-09-06" for year in self.full_range))

        obs_dts = (
            "2026-09-07",
            "2037-09-07",
            "2043-09-07",
            "2048-09-07",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        obs_dts = (
            "2022-12-27",
            "2033-12-27",
            "2039-12-27",
            "2044-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        obs_dts = (
            "2021-12-27",
            "2027-12-27",
            "2032-12-27",
            "2038-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_swaziland_deprecation_warning(self):
        warnings.simplefilter("default")
        with self.assertWarns(DeprecationWarning):
            holidays.Swaziland()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.mauritania import Mauritania
from tests.common import CommonCountryTests


class TestMauritania(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Mauritania, years=range(1950, 2050))

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1950, 2050)))

    def test_labor_day(self):
        self.assertHolidayName("Labor Day", (f"{year}-05-01" for year in range(1950, 2050)))

    def test_africa_day(self):
        self.assertHolidayName("Africa Day", (f"{year}-05-25" for year in range(1950, 2050)))

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-11-28" for year in range(1960, 2050)))
        self.assertNoHolidayName(name, range(1950, 1960))

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-04-21", "Eid al-Fitr (estimated)"),
            ("2023-04-22", "Eid al-Fitr (estimated)"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-25", "Africa Day"),
            ("2023-06-28", "Eid al-Adha (estimated)"),
            ("2023-06-29", "Eid al-Adha (estimated)"),
            ("2023-07-19", "Islamic New Year (estimated)"),
            ("2023-09-27", "Mawlid al-Nabi (estimated)"),
            ("2023-11-28", "Independence Day"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-04-10", "Eid al-Fitr (estimated)"),
            ("2024-04-11", "Eid al-Fitr (estimated)"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-25", "Africa Day"),
            ("2024-06-16", "Eid al-Adha (estimated)"),
            ("2024-06-17", "Eid al-Adha (estimated)"),
            ("2024-07-07", "Islamic New Year (estimated)"),
            ("2024-09-15", "Mawlid al-Nabi (estimated)"),
            ("2024-11-28", "Independence Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.peru import Peru
from tests.common import CommonCountryTests


class TestPeru(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Peru, years=range(1950, 2050))

    def test_2019(self):
        self.assertHolidayDatesInYear(
            2019,
            "2019-01-01",
            "2019-04-18",
            "2019-04-19",
            "2019-04-21",
            "2019-05-01",
            "2019-06-29",
            "2019-07-28",
            "2019-07-29",
            "2019-08-30",
            "2019-10-08",
            "2019-11-01",
            "2019-12-08",
            "2019-12-25",
        )

    def test_2022(self):
        self.assertHolidayDatesInYear(
            2022,
            "2022-01-01",
            "2022-04-14",
            "2022-04-15",
            "2022-04-17",
            "2022-05-01",
            "2022-06-29",
            "2022-07-28",
            "2022-07-29",
            "2022-08-06",
            "2022-08-30",
            "2022-10-08",
            "2022-11-01",
            "2022-12-08",
            "2022-12-09",
            "2022-12-25",
        )

    def test_battle_of_arica_and_flag_day(self):
        name = "Batalla de Arica y Día de la Bandera"
        self.assertHolidayName(name, (f"{year}-06-07" for year in range(2024, 2050)))
        self.assertNoHolidayName(name, range(1950, 2024))

    def test_peruvian_air_force_day(self):
        name = "Día de la Fuerza Aérea del Perú"
        self.assertHolidayName(name, (f"{year}-07-23" for year in range(2023, 2050)))
        self.assertNoHolidayName(name, range(1950, 2023))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-17", "Domingo de Resurrección"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-06-29", "San Pedro y San Pablo"),
            ("2022-07-28", "Día de la Independencia"),
            ("2022-07-29", "Día de la Gran Parada Militar"),
            ("2022-08-06", "Batalla de Junín"),
            ("2022-08-30", "Santa Rosa de Lima"),
            ("2022-10-08", "Combate de Angamos"),
            ("2022-11-01", "Todos Los Santos"),
            ("2022-12-08", "Inmaculada Concepción"),
            ("2022-12-09", "Batalla de Ayacucho"),
            ("2022-12-25", "Navidad del Señor"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-01", "Labor Day"),
            ("2022-06-29", "Saint Peter and Saint Paul's Day"),
            ("2022-07-28", "Independence Day"),
            ("2022-07-29", "Great Military Parade Day"),
            ("2022-08-06", "Battle of Junín Day"),
            ("2022-08-30", "Rose of Lima Day"),
            ("2022-10-08", "Battle of Angamos Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-08", "Immaculate Conception"),
            ("2022-12-09", "Battle of Ayacucho Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-05-01", "День праці"),
            ("2022-06-29", "День Святих Петра і Павла"),
            ("2022-07-28", "День незалежності"),
            ("2022-07-29", "День Великого військового параду"),
            ("2022-08-06", "День битви під Хуніном"),
            ("2022-08-30", "День Святої Рози Лімської"),
            ("2022-10-08", "День битви під Ангамосом"),
            ("2022-11-01", "День усіх святих"),
            ("2022-12-08", "Непорочне зачаття Діви Марії"),
            ("2022-12-09", "День битви при Аякучо"),
            ("2022-12-25", "Різдво Христове"),
        )
